#
# Copyright 2011-2014 Blender Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# <pep8 compliant>

import bpy

from bpy.app.handlers import persistent


def check_is_new_shading_ntree(node_tree):
    for node in node_tree.nodes:
        # If material has any node with ONLY new shading system
        # compatibility then it's considered a Cycles material
        # and versioning code would need to perform on it.
        #
        # We can not check for whether NEW_SHADING in compatibility
        # because some nodes could have compatibility with both old
        # and new shading system and they can't be used for any
        # decision here.
        if node.shading_compatibility == {'NEW_SHADING'}:
            return True

        # If node is only compatible with old shading system
        # then material can not be Cycles material and we
        # can stopiterating nodes now.
        if node.shading_compatibility == {'OLD_SHADING'}:
            return False
    return False


def check_is_new_shading_material(material):
    if not material.node_tree:
        return False
    return check_is_new_shading_ntree(material.node_tree)


def check_is_new_shading_world(world):
    if not world.node_tree:
        return False
    return check_is_new_shading_ntree(world.node_tree)


def check_is_new_shading_lamp(lamp):
    if not lamp.node_tree:
        return False
    return check_is_new_shading_ntree(lamp.node_tree)


def foreach_notree_node(nodetree, callback, traversed):
    if nodetree in traversed:
        return
    traversed.add(nodetree)
    for node in nodetree.nodes:
        callback(node)
        if node.bl_idname == 'ShaderNodeGroup':
            foreach_notree_node(node.node_tree, callback, traversed)


def foreach_cycles_node(callback):
    traversed = set()
    for material in bpy.data.materials:
        if check_is_new_shading_material(material):
                foreach_notree_node(material.node_tree,
                                    callback,
                                    traversed)
    for world in bpy.data.worlds:
        if check_is_new_shading_world(world):
                foreach_notree_node(world.node_tree,
                                    callback,
                                    traversed)
    for lamp in bpy.data.lamps:
        if check_is_new_shading_world(lamp):
                foreach_notree_node(lamp.node_tree,
                                    callback,
                                    traversed)


def mapping_node_order_flip(node):
    """
    Flip euler order of mapping shader node
    """
    if node.bl_idname == 'ShaderNodeMapping':
        rot = node.rotation.copy()
        rot.order = 'ZYX'
        quat = rot.to_quaternion()
        node.rotation = quat.to_euler('XYZ')


def vector_curve_node_remap(node):
    """
    Remap values of vector curve node from normalized to absolute values
    """
    if node.bl_idname == 'ShaderNodeVectorCurve':
        node.mapping.use_clip = False
        for curve in node.mapping.curves:
            for point in curve.points:
                point.location.x = (point.location.x * 2.0) - 1.0
                point.location.y = (point.location.y - 0.5) * 2.0
        node.mapping.update()


def custom_bake_remap(scene):
    """
    Remap bake types into the new types and set the flags accordingly
    """
    bake_lookup = (
        'COMBINED',
        'AO',
        'SHADOW',
        'NORMAL',
        'UV',
        'EMIT',
        'ENVIRONMENT',
        'DIFFUSE_DIRECT',
        'DIFFUSE_INDIRECT',
        'DIFFUSE_COLOR',
        'GLOSSY_DIRECT',
        'GLOSSY_INDIRECT',
        'GLOSSY_COLOR',
        'TRANSMISSION_DIRECT',
        'TRANSMISSION_INDIRECT',
        'TRANSMISSION_COLOR',
        'SUBSURFACE_DIRECT',
        'SUBSURFACE_INDIRECT',
        'SUBSURFACE_COLOR')

    diffuse_direct_idx = bake_lookup.index('DIFFUSE_DIRECT')

    cscene = scene.cycles

    # Old bake type
    bake_type_idx = cscene.get("bake_type")

    if bake_type_idx is None:
        cscene.bake_type = 'COMBINED'
        return

    # File doesn't need versioning
    if bake_type_idx < diffuse_direct_idx:
        return

    # File needs versioning
    bake_type = bake_lookup[bake_type_idx]
    cscene.bake_type, end = bake_type.split('_')

    if end == 'DIRECT':
        scene.render.bake.use_pass_indirect = False
        scene.render.bake.use_pass_color = False

    elif end == 'INDIRECT':
        scene.render.bake.use_pass_direct = False
        scene.render.bake.use_pass_color = False

    elif end == 'COLOR':
        scene.render.bake.use_pass_direct = False
        scene.render.bake.use_pass_indirect = False


@persistent
def do_versions(self):
    if bpy.context.user_preferences.version <= (2, 78, 1):
        prop = bpy.context.user_preferences.addons[__package__].preferences
        system = bpy.context.user_preferences.system
        if not prop.is_property_set("compute_device_type"):
            # Device might not currently be available so this can fail
            try:
                if system.legacy_compute_device_type == 1:
                    prop.compute_device_type = 'OPENCL'
                elif system.legacy_compute_device_type == 2:
                    prop.compute_device_type = 'CUDA'
                else:
                    prop.compute_device_type = 'NONE'
            except:
                pass

            # Init device list for UI
            prop.get_devices()

    # We don't modify startup file because it assumes to
    # have all the default values only.
    if not bpy.data.is_saved:
        return

    # Clamp Direct/Indirect separation in 270
    if bpy.data.version <= (2, 70, 0):
        for scene in bpy.data.scenes:
            cscene = scene.cycles
            sample_clamp = cscene.get("sample_clamp", False)
            if (sample_clamp and
                not cscene.is_property_set("sample_clamp_direct") and
                not cscene.is_property_set("sample_clamp_indirect")):

                cscene.sample_clamp_direct = sample_clamp
                cscene.sample_clamp_indirect = sample_clamp

    # Change of Volume Bounces in 271
    if bpy.data.version <= (2, 71, 0):
        for scene in bpy.data.scenes:
            cscene = scene.cycles
            if not cscene.is_property_set("volume_bounces"):
                cscene.volume_bounces = 1

    # Caustics Reflective/Refractive separation in 272
    if bpy.data.version <= (2, 72, 0):
        for scene in bpy.data.scenes:
            cscene = scene.cycles
            if (cscene.get("no_caustics", False) and
                not cscene.is_property_set("caustics_reflective") and
                not cscene.is_property_set("caustics_refractive")):

                cscene.caustics_reflective = False
                cscene.caustics_refractive = False

    # Euler order was ZYX in previous versions.
    if bpy.data.version <= (2, 73, 4):
        foreach_cycles_node(mapping_node_order_flip)

    if bpy.data.version <= (2, 76, 5):
        foreach_cycles_node(vector_curve_node_remap)

    # Baking types changed
    if bpy.data.version <= (2, 76, 6):
        for scene in bpy.data.scenes:
            custom_bake_remap(scene)

    # Several default changes for 2.77
    if bpy.data.version <= (2, 76, 8):
        for scene in bpy.data.scenes:
            cscene = scene.cycles

            # Samples
            if not cscene.is_property_set("samples"):
                cscene.samples = 10

            # Preview Samples
            if not cscene.is_property_set("preview_samples"):
                cscene.preview_samples = 10

            # Filter
            if not cscene.is_property_set("filter_type"):
                cscene.pixel_filter_type = 'GAUSSIAN'

            # Tile Order
            if not cscene.is_property_set("tile_order"):
                cscene.tile_order = 'CENTER'

        for lamp in bpy.data.lamps:
            clamp = lamp.cycles

            # MIS
            if not clamp.is_property_set("use_multiple_importance_sampling"):
                clamp.use_multiple_importance_sampling = False

        for mat in bpy.data.materials:
            cmat = mat.cycles

            # Volume Sampling
            if not cmat.is_property_set("volume_sampling"):
                cmat.volume_sampling = 'DISTANCE'

    if bpy.data.version <= (2, 76, 9):
        for world in bpy.data.worlds:
            cworld = world.cycles

            # World MIS
            if not cworld.is_property_set("sample_as_light"):
                cworld.sample_as_light = False

            # World MIS Samples
            if not cworld.is_property_set("samples"):
                cworld.samples = 4

            # World MIS Resolution
            if not cworld.is_property_set("sample_map_resolution"):
                cworld.sample_map_resolution = 256

    if bpy.data.version <= (2, 76, 10):
        for scene in bpy.data.scenes:
            cscene = scene.cycles
            if cscene.is_property_set("filter_type"):
                if not cscene.is_property_set("pixel_filter_type"):
                    cscene.pixel_filter_type = cscene.filter_type
                if cscene.filter_type == 'BLACKMAN_HARRIS':
                    cscene.filter_type = 'GAUSSIAN'

    if bpy.data.version <= (2, 78, 2):
        for scene in bpy.data.scenes:
            cscene = scene.cycles
            if not cscene.is_property_set("light_sampling_threshold"):
                cscene.light_sampling_threshold = 0.0



import os
import bpy
import bpy.utils.previews

mesh_check_icon_collections = {}
mesh_check_icons_loaded = False


def load_icons():
    global mesh_check_icon_collections
    global mesh_check_icons_loaded

    if mesh_check_icons_loaded:
        return mesh_check_icon_collections["main"]

    custom_icons = bpy.utils.previews.new()

    icons_dir = os.path.join(os.path.dirname(__file__))

    custom_icons.load("ngons", os.path.join(icons_dir, "ngon.png"), 'IMAGE')
    custom_icons.load("triangles", os.path.join(icons_dir, "triangle.png"), 'IMAGE')

    mesh_check_icon_collections["main"] = custom_icons
    mesh_check_icons_loaded = True

    return mesh_check_icon_collections["main"]


def clear_icons():
    global mesh_check_icons_loaded
    for icon in mesh_check_icon_collections.values():
        bpy.utils.previews.remove(icon)
    mesh_check_icon_collections.clear()
    mesh_check_icons_loaded = False



# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8 compliant>

bl_info = {
    "name": "Hotkey: 'Ctrl Alt S ",
    "description": "Switch Editor Type Menu",
    "author": "saidenka",
    "version": (0, 1, 0),
    "blender": (2, 77, 0),
    "location": "All Editors",
    "warning": "",
    "wiki_url": "",
    "category": "Editor Switch Pie"
    }

import bpy
from bpy.types import (
        Menu,
        Operator,
        )
from bpy.props import (
        StringProperty,
        )


class AreaPieMenu(Menu):
    bl_idname = "INFO_MT_window_pie"
    bl_label = "Pie Menu"
    bl_description = "Window Pie Menus"

    def draw(self, context):
        self.layout.operator(AreaTypePieOperator.bl_idname, icon="PLUGIN")


class AreaTypePieOperator(Operator):
    bl_idname = "wm.area_type_pie_operator"
    bl_label = "Editor Type"
    bl_description = "This is pie menu of editor type change"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        bpy.ops.wm.call_menu_pie(name=AreaPieEditor.bl_idname)

        return {'FINISHED'}


class AreaPieEditor(Menu):
    bl_idname = "pie.editor"
    bl_label = "Editor Switch"

    def draw(self, context):
        layout = self.layout
        pie = layout.menu_pie()
        # 4 - LEFT
        pie.operator(SetAreaType.bl_idname, text="Text Editor", icon="TEXT").types = "TEXT_EDITOR"
        # 6 - RIGHT
        pie.menu(AreaTypePieAnim.bl_idname, text="Animation Editors", icon="ACTION")
        # 2 - BOTTOM
        pie.operator(SetAreaType.bl_idname, text="Property", icon="BUTS").types = "PROPERTIES"
        # 8 - TOP
        pie.operator(SetAreaType.bl_idname, text="3D View", icon="MESH_CUBE").types = "VIEW_3D"
        # 7 - TOP - LEFT
        pie.operator(SetAreaType.bl_idname, text="UV/Image Editor", icon="IMAGE_COL").types = "IMAGE_EDITOR"
        # 9 - TOP - RIGHT
        pie.operator(SetAreaType.bl_idname, text="Node Editor", icon="NODETREE").types = "NODE_EDITOR"
        # 1 - BOTTOM - LEFT
        pie.operator(SetAreaType.bl_idname, text="Outliner", icon="OOPS").types = "OUTLINER"
        # 3 - BOTTOM - RIGHT
        pie.menu(AreaTypePieOther.bl_idname, text="More Editors", icon="QUESTION")


class AreaTypePieOther(Menu):
    bl_idname = "INFO_MT_window_pie_area_type_other"
    bl_label = "Editor Type (other)"
    bl_description = "Is pie menu change editor type (other)"

    def draw(self, context):
        # 4 - LEFT
        self.layout.operator(SetAreaType.bl_idname, text="Logic Editor", icon="LOGIC").types = "LOGIC_EDITOR"
        # 6 - RIGHT
        self.layout.operator(SetAreaType.bl_idname, text="File Browser", icon="FILESEL").types = "FILE_BROWSER"
        # 2 - BOTTOM
        self.layout.operator(SetAreaType.bl_idname, text="Python Console", icon="CONSOLE").types = "CONSOLE"
        # 8 - TOP
        # 7 - TOP - LEFT
        self.layout.operator(SetAreaType.bl_idname, text="User Preferences",
                             icon="PREFERENCES").types = "USER_PREFERENCES"
        # 9 - TOP - RIGHT
        self.layout.operator(SetAreaType.bl_idname, text="Info", icon="INFO").types = "INFO"
        # 1 - BOTTOM - LEFT
        # 3 - BOTTOM - RIGHT


class SetAreaType(Operator):
    bl_idname = "wm.set_area_type"
    bl_label = "Change Editor Type"
    bl_description = "Change Editor Type"
    bl_options = {'REGISTER'}

    types = StringProperty(name="Area Type")

    def execute(self, context):
        context.area.type = self.types
        return {'FINISHED'}


class AreaTypePieAnim(Menu):
    bl_idname = "INFO_MT_window_pie_area_type_anim"
    bl_label = "Editor Type (Animation)"
    bl_description = "Menu for changing editor type (animation related)"

    def draw(self, context):
        # 4 - LEFT
        self.layout.operator(SetAreaType.bl_idname, text="NLA Editor", icon="NLA").types = "NLA_EDITOR"
        # 6 - RIGHT
        self.layout.operator(SetAreaType.bl_idname, text="DopeSheet", icon="ACTION").types = "DOPESHEET_EDITOR"
        # 2 - BOTTOM
        self.layout.operator(SetAreaType.bl_idname, text="Graph Editor", icon="IPO").types = "GRAPH_EDITOR"
        # 8 - TOP
        self.layout.operator(SetAreaType.bl_idname, text="Timeline", icon="TIME").types = "TIMELINE"
        # 7 - TOP - LEFT
        self.layout.operator(SetAreaType.bl_idname,
                             text="Video Sequence Editor", icon="SEQUENCE").types = "SEQUENCE_EDITOR"
        # 9 - TOP - RIGHT
        self.layout.operator(SetAreaType.bl_idname,
                             text="Video Clip Editor", icon="RENDER_ANIMATION").types = "CLIP_EDITOR"
        # 1 - BOTTOM - LEFT
        self.layout.operator("wm.call_menu_pie", text="Back", icon="BACK").name = AreaPieEditor.bl_idname
        # 3 - BOTTOM - RIGHT


classes = (
    AreaPieMenu,
    AreaTypePieOperator,
    AreaPieEditor,
    AreaTypePieOther,
    SetAreaType,
    AreaTypePieAnim,
    )

addon_keymaps = []


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

    wm = bpy.context.window_manager
    if wm.keyconfigs.addon:
        # Snapping
        km = wm.keyconfigs.addon.keymaps.new(name='Window')
        kmi = km.keymap_items.new('wm.call_menu_pie', 'S', 'PRESS', ctrl=True, alt=True)
        kmi.properties.name = "pie.editor"
        addon_keymaps.append((km, kmi))


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        for km, kmi in addon_keymaps:
            km.keymap_items.remove(kmi)
    addon_keymaps.clear()


if __name__ == "__main__":
    register()



# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8 compliant>

import bpy

from bpy.types import Node, ShaderNodeTree, CompositorNodeTree, TextureNodeTree#, NodeSocket


from bpy.props import (
        StringProperty,
        BoolProperty,
        IntProperty,
        FloatProperty,
        FloatVectorProperty,
        EnumProperty,
        #PointerProperty,
        #CollectionProperty,
        )



############### object

class ObjectNodeTree(bpy.types.NodeTree):
    '''Povray Material Nodes'''

    bl_idname = 'ObjectNodeTree'
    bl_label = 'Povray Object Nodes'
    bl_icon = 'PLUGIN'

    @classmethod
    def poll(cls, context):
        return context.scene.render.engine == 'POVRAY_RENDER'

    @classmethod
    def get_from_context(cls, context):
        ob = context.active_object
        if ob and ob.type not in {'LAMP'}:
            ma = ob.active_material
            if ma is not None:
                nt_name = ma.node_tree
                if nt_name != '':
                    return nt_name, ma, ma
        return (None, None, None)

    def update(self):
        self.refresh = True
################### output #############################################################################################

class PovrayOutputNode(Node, ObjectNodeTree):
    '''Output'''
    bl_idname = 'PovrayOutputNode'
    bl_label = 'Output'
    bl_icon = 'SOUND'

    def init(self, context):

        self.inputs.new('PovraySocketTexture', "Texture")

    def draw_buttons(self, context, layout):

        ob=context.object
        layout.prop(ob.pov, "object_ior",slider=True)

    def draw_buttons_ext(self, context, layout):

        ob=context.object
        layout.prop(ob.pov, "object_ior",slider=True)

    def draw_label(self):
        return "Output"



################### material ###########################################################################################
class PovrayTextureNode(Node, ObjectNodeTree):
    '''Texture'''
    bl_idname = 'PovrayTextureNode'
    bl_label = 'Simple texture'
    bl_icon = 'SOUND'

    def init(self, context):

        color=self.inputs.new('PovraySocketColor', "Pigment")
        color.default_value=(1,1,1)
        normal=self.inputs.new('NodeSocketFloat', "Normal")
        normal.hide_value=True
        finish=self.inputs.new('NodeSocketVector', "Finish")
        finish.hide_value=True

        self.outputs.new('PovraySocketTexture', "Texture")

    def draw_label(self):
        return "Simple texture"

class PovrayFinishNode(Node, ObjectNodeTree):
    '''Finish'''
    bl_idname = 'PovrayFinishNode'
    bl_label = 'Finish'
    bl_icon = 'SOUND'

    def init(self, context):

        self.inputs.new('PovraySocketFloat_0_1', "Emission")
        ambient=self.inputs.new('NodeSocketVector', "Ambient")
        ambient.hide_value=True
        diffuse=self.inputs.new('NodeSocketVector', "Diffuse")
        diffuse.hide_value=True
        specular=self.inputs.new('NodeSocketVector', "Highlight")
        specular.hide_value=True
        mirror=self.inputs.new('NodeSocketVector', "Mirror")
        mirror.hide_value=True
        iridescence=self.inputs.new('NodeSocketVector', "Iridescence")
        iridescence.hide_value=True
        subsurface=self.inputs.new('NodeSocketVector', "Translucency")
        subsurface.hide_value=True
        self.outputs.new('NodeSocketVector', "Finish")

    def draw_label(self):
        return "Finish"

class PovrayDiffuseNode(Node, ObjectNodeTree):
    '''Diffuse'''
    bl_idname = 'PovrayDiffuseNode'
    bl_label = 'Diffuse'
    bl_icon = 'SOUND'

    def init(self, context):

        intensity=self.inputs.new('PovraySocketFloat_0_1', "Intensity")
        intensity.default_value=0.8
        albedo=self.inputs.new('NodeSocketBool', "Albedo")
        albedo.default_value=False
        brilliance=self.inputs.new('PovraySocketFloat_0_10', "Brilliance")
        brilliance.default_value=1.8
        self.inputs.new('PovraySocketFloat_0_1', "Crand")
        self.outputs.new('NodeSocketVector', "Diffuse")

    def draw_label(self):
        return "Diffuse"

class PovrayPhongNode(Node, ObjectNodeTree):
    '''Phong'''
    bl_idname = 'PovrayPhongNode'
    bl_label = 'Phong'
    bl_icon = 'SOUND'

    def init(self, context):

        albedo=self.inputs.new('NodeSocketBool', "Albedo")
        intensity=self.inputs.new('PovraySocketFloat_0_1', "Intensity")
        intensity.default_value=0.8
        phong_size=self.inputs.new('PovraySocketInt_0_256', "Size")
        phong_size.default_value=60
        metallic=self.inputs.new('PovraySocketFloat_0_1', "Metallic")

        self.outputs.new('NodeSocketVector', "Phong")

    def draw_label(self):
        return "Phong"

class PovraySpecularNode(Node, ObjectNodeTree):
    '''Specular'''
    bl_idname = 'PovraySpecularNode'
    bl_label = 'Specular'
    bl_icon = 'SOUND'

    def init(self, context):

        albedo=self.inputs.new('NodeSocketBool', "Albedo")
        intensity=self.inputs.new('PovraySocketFloat_0_1', "Intensity")
        intensity.default_value=0.8
        roughness=self.inputs.new('PovraySocketFloat_0_1', "Roughness")
        roughness.default_value=0.02
        metallic=self.inputs.new('PovraySocketFloat_0_1', "Metallic")

        self.outputs.new('NodeSocketVector', "Specular")

    def draw_label(self):
        return "Specular"

class PovrayMirrorNode(Node, ObjectNodeTree):
    '''Mirror'''
    bl_idname = 'PovrayMirrorNode'
    bl_label = 'Mirror'
    bl_icon = 'SOUND'

    def init(self, context):

        color=self.inputs.new('PovraySocketColor', "Color")
        color.default_value=(1,1,1)
        metallic=self.inputs.new('PovraySocketFloat_0_1', "Metallic")
        metallic.default_value=1.0
        exponent=self.inputs.new('PovraySocketFloat_0_1', "Exponent")
        exponent.default_value=1.0
        self.inputs.new('PovraySocketFloat_0_1', "Falloff")
        self.inputs.new('NodeSocketBool', "Fresnel")
        self.inputs.new('NodeSocketBool', "Conserve energy")
        self.outputs.new('NodeSocketVector', "Mirror")

    def draw_label(self):
        return "Mirror"

class PovrayAmbientNode(Node, ObjectNodeTree):
    '''Ambient'''
    bl_idname = 'PovrayAmbientNode'
    bl_label = 'Ambient'
    bl_icon = 'SOUND'

    def init(self, context):

        self.inputs.new('PovraySocketColor', "Ambient")

        self.outputs.new('NodeSocketVector', "Ambient")

    def draw_label(self):
        return "Ambient"

class PovrayIridescenceNode(Node, ObjectNodeTree):
    '''Iridescence'''
    bl_idname = 'PovrayIridescenceNode'
    bl_label = 'Iridescence'
    bl_icon = 'SOUND'

    def init(self, context):

        amount=self.inputs.new('NodeSocketFloat', "Amount")
        amount.default_value=0.25
        thickness=self.inputs.new('NodeSocketFloat', "Thickness")
        thickness.default_value=1
        self.inputs.new('NodeSocketFloat', "Turbulence")

        self.outputs.new('NodeSocketVector', "Iridescence")

    def draw_label(self):
        return "Iridescence"

class PovraySubsurfaceNode(Node, ObjectNodeTree):
    '''Subsurface'''
    bl_idname = 'PovraySubsurfaceNode'
    bl_label = 'Subsurface'
    bl_icon = 'SOUND'

    def init(self, context):

        translucency=self.inputs.new('NodeSocketColor', "Translucency")
        translucency.default_value=(0,0,0,1)
        energy=self.inputs.new('PovraySocketInt_0_256', "Energy")
        energy.default_value=20
        self.outputs.new('NodeSocketVector', "Translucency")

    def draw_buttons(self, context, layout):
        scene=context.scene
        layout.prop(scene.pov, "sslt_enable",text="SSLT")


    def draw_buttons_ext(self, context, layout):
        scene=context.scene
        layout.prop(scene.pov, "sslt_enable",text="SSLT")

    def draw_label(self):
        return "Subsurface"

#####################################################################################################

class PovrayMappingNode(Node, ObjectNodeTree):
    '''Mapping'''
    bl_idname = 'PovrayMappingNode'
    bl_label = 'Mapping'
    bl_icon = 'SOUND'

    warp_type = EnumProperty(
            name="Warp Types",
            description="Select the type of warp",
            items=( ('cubic', "Cubic", ""),  ('cylindrical', "Cylindrical", ""),('planar', "Planar", ""),
                    ('spherical', "Spherical", ""),('toroidal', "Toroidal", ""),
                    ('uv_mapping', "UV", ""),
                    ('NONE', "None", "No indentation")),
            default='NONE')

    warp_orientation = EnumProperty(
            name="Warp Orientation",
            description="Select the orientation of warp",
            items=(('x', "X", ""), ('y', "Y", ""), ('z', "Z", "")),
            default='y')

    warp_dist_exp = FloatProperty(
            name="Distance exponent",
            description="Distance exponent",
            min=0.0, max=100.0, default=1.0)

    warp_tor_major_radius = FloatProperty(
            name="Major radius",
            description="Torus is distance from major radius",
            min=0.0, max=5.0, default=1.0)

    def init(self, context):
        self.outputs.new('NodeSocketVector', "Mapping")

    def draw_buttons(self, context, layout):

        column=layout.column()
        column.prop(self,"warp_type",text="Warp type")
        if self.warp_type in {'toroidal','spherical','cylindrical','planar'}:
            column.prop(self,"warp_orientation",text="Orientation")
            column.prop(self,"warp_dist_exp",text="Exponent")
        if self.warp_type=='toroidal':
            column.prop(self,"warp_tor_major_radius",text="Major R")

    def draw_buttons_ext(self, context, layout):

        column=layout.column()
        column.prop(self,"warp_type",text="Warp type")
        if self.warp_type in {'toroidal','spherical','cylindrical','planar'}:
            column.prop(self,"warp_orientation",text="Orientation")
            column.prop(self,"warp_dist_exp",text="Exponent")
        if self.warp_type=='toroidal':
            column.prop(self,"warp_tor_major_radius",text="Major R")

    def draw_label(self):
        return "Mapping"

class PovrayMultiplyNode(Node, ObjectNodeTree):
    '''Multiply'''
    bl_idname = 'PovrayMultiplyNode'
    bl_label = 'Multiply'
    bl_icon = 'SOUND'

    amount_x = FloatProperty(
            name="X",
            description="Number of repeats",
            min=1.0, max=10000.0, default=1.0)

    amount_y = FloatProperty(
            name="Y",
            description="Number of repeats",
            min=1.0, max=10000.0, default=1.0)

    amount_z = FloatProperty(
            name="Z",
            description="Number of repeats",
            min=1.0, max=10000.0, default=1.0)


    def init(self, context):
        self.outputs.new('NodeSocketVector', "Amount")

    def draw_buttons(self, context, layout):

        column=layout.column()
        column.label("Amount")
        row=column.row(align=True)
        row.prop(self,"amount_x")
        row.prop(self,"amount_y")
        row.prop(self,"amount_z")

    def draw_buttons_ext(self, context, layout):

        column=layout.column()
        column.label("Amount")
        row=column.row(align=True)
        row.prop(self,"amount_x")
        row.prop(self,"amount_y")
        row.prop(self,"amount_z")

    def draw_label(self):
        return "Multiply"

class PovrayTransformNode(Node, ObjectNodeTree):
    '''Transform'''
    bl_idname = 'PovrayTransformNode'
    bl_label = 'Transform'
    bl_icon = 'SOUND'

    def init(self, context):

        self.inputs.new('PovraySocketFloatUnlimited', "Translate x")
        self.inputs.new('PovraySocketFloatUnlimited', "Translate y")
        self.inputs.new('PovraySocketFloatUnlimited', "Translate z")
        self.inputs.new('PovraySocketFloatUnlimited', "Rotate x")
        self.inputs.new('PovraySocketFloatUnlimited', "Rotate y")
        self.inputs.new('PovraySocketFloatUnlimited', "Rotate z")
        sX = self.inputs.new('PovraySocketFloatUnlimited', "Scale x")
        sX.default_value = 1.0
        sY = self.inputs.new('PovraySocketFloatUnlimited', "Scale y")
        sY.default_value = 1.0
        sZ = self.inputs.new('PovraySocketFloatUnlimited', "Scale z")
        sZ.default_value = 1.0

        self.outputs.new('NodeSocketVector', "Transform")

    def draw_label(self):
        return "Transform"

class PovrayValueNode(Node, ObjectNodeTree):
    '''Value'''
    bl_idname = 'PovrayValueNode'
    bl_label = 'Value'
    bl_icon = 'SOUND'

    def init(self, context):

        self.outputs.new('PovraySocketUniversal', "Value")

    def draw_label(self):
        return "Value"

class PovrayModifierNode(Node, ObjectNodeTree):
    '''Modifier'''
    bl_idname = 'PovrayModifierNode'
    bl_label = 'Modifier'
    bl_icon = 'SOUND'

    def init(self, context):

        turb_x=self.inputs.new('PovraySocketFloat_0_10', "Turb X")
        turb_x.default_value=0.1
        turb_y=self.inputs.new('PovraySocketFloat_0_10', "Turb Y")
        turb_y.default_value=0.1
        turb_z=self.inputs.new('PovraySocketFloat_0_10', "Turb Z")
        turb_z.default_value=0.1
        octaves=self.inputs.new('PovraySocketInt_1_9', "Octaves")
        octaves.default_value=1
        lambat=self.inputs.new('PovraySocketFloat_0_10', "Lambda")
        lambat.default_value=2.0
        omega=self.inputs.new('PovraySocketFloat_0_10', "Omega")
        omega.default_value=0.5
        freq=self.inputs.new('PovraySocketFloat_0_10', "Frequency")
        freq.default_value=2.0
        self.inputs.new('PovraySocketFloat_0_10', "Phase")

        self.outputs.new('NodeSocketVector', "Modifier")

    def draw_label(self):
        return "Modifier"

class PovrayPigmentNode(Node, ObjectNodeTree):
    '''Pigment'''
    bl_idname = 'PovrayPigmentNode'
    bl_label = 'Color'

    def init(self, context):

        color = self.inputs.new('PovraySocketColor', "Color")
        color.default_value = (1,1,1)
        povfilter = self.inputs.new('PovraySocketFloat_0_1', "Filter")
        transmit = self.inputs.new('PovraySocketFloat_0_1', "Transmit")
        self.outputs.new('NodeSocketColor', "Pigment")

    def draw_label(self):
        return "Color"

class PovrayColorImageNode(Node, ObjectNodeTree):
    '''ColorImage'''
    bl_idname = 'PovrayColorImageNode'
    bl_label = 'Image map'

    map_type = bpy.props.EnumProperty(
            name="Map type",
            description="",
            items=( ('uv_mapping', "UV", ""),
                    ('0', "Planar", "Default planar mapping"),
                    ('1', "Spherical", "Spherical mapping"),
                    ('2', "Cylindrical", "Cylindrical mapping"),
                    ('5', "Torroidal", "Torus or donut shaped mapping")),
            default='0')
    image = StringProperty(maxlen=1024) # , subtype="FILE_PATH"
    interpolate = EnumProperty(
            name="Interpolate",
            description="Adding the interpolate keyword can smooth the jagged look of a bitmap",
            items=(
                ('2', "Bilinear", "Gives bilinear interpolation"),
                ('4', "Normalized", "Gives normalized distance"),
            ),
            default='2')
    premultiplied = BoolProperty(default=False)
    once = BoolProperty(description="Not to repeat", default=False)

    def init(self, context):

        gamma=self.inputs.new('PovraySocketFloat_000001_10', "Gamma")
        gamma.default_value=2.0
        transmit=self.inputs.new('PovraySocketFloat_0_1', "Transmit")
        povfilter=self.inputs.new('PovraySocketFloat_0_1', "Filter")
        mapping=self.inputs.new('NodeSocketVector', "Mapping")
        mapping.hide_value=True
        transform=self.inputs.new('NodeSocketVector', "Transform")
        transform.hide_value=True
        modifier=self.inputs.new('NodeSocketVector', "Modifier")
        modifier.hide_value=True

        self.outputs.new('NodeSocketColor', "Pigment")

    def draw_buttons(self, context, layout):

        column=layout.column()
        im=None
        for image in bpy.data.images:
            if image.name == self.image:
                im=image
        split = column.split(percentage=0.8,align=True)
        split.prop_search(self,"image",context.blend_data,"images",text="")
        split.operator("pov.imageopen",text="",icon="FILESEL")
        if im is not None:
            column.prop(im,"source",text="")
        column.prop(self,"map_type",text="")
        column.prop(self,"interpolate",text="")
        row=column.row()
        row.prop(self,"premultiplied",text="Premul")
        row.prop(self,"once",text="Once")

    def draw_buttons_ext(self, context, layout):

        column=layout.column()
        im=None
        for image in bpy.data.images:
            if image.name == self.image:
                im=image
        split = column.split(percentage=0.8,align=True)
        split.prop_search(self,"image",context.blend_data,"images",text="")
        split.operator("pov.imageopen",text="",icon="FILESEL")
        if im is not None:
            column.prop(im,"source",text="")
        column.prop(self,"map_type",text="")
        column.prop(self,"interpolate",text="")
        row=column.row()
        row.prop(self,"premultiplied",text="Premul")
        row.prop(self,"once",text="Once")

    def draw_label(self):
        return "Image map"

class PovrayBumpMapNode(Node, ObjectNodeTree):
    '''BumpMap'''
    bl_idname = 'PovrayBumpMapNode'
    bl_label = 'Bump map'
    bl_icon = 'SOUND'

    map_type = bpy.props.EnumProperty(
            name="Map type",
            description="",
            items=(
                ('uv_mapping', "UV", ""),
                ('0', "Planar", "Default planar mapping"),
                ('1', "Spherical", "Spherical mapping"),
                ('2', "Cylindrical", "Cylindrical mapping"),
                ('5', "Torroidal", "Torus or donut shaped mapping")
            ),
            default='0')
    image = StringProperty(maxlen=1024) # , subtype="FILE_PATH"
    interpolate = EnumProperty(
            name="Interpolate",
            description="Adding the interpolate keyword can smooth the jagged look of a bitmap",
            items=(
                ('2', "Bilinear", "Gives bilinear interpolation"),
                ('4', "Normalized", "Gives normalized distance"),
            ),
            default='2')
    once = BoolProperty(description="Not to repeat", default=False)

    def init(self, context):

        self.inputs.new('PovraySocketFloat_0_10', "Normal")
        mapping=self.inputs.new('NodeSocketVector', "Mapping")
        mapping.hide_value=True
        transform=self.inputs.new('NodeSocketVector', "Transform")
        transform.hide_value=True
        modifier=self.inputs.new('NodeSocketVector', "Modifier")
        modifier.hide_value=True

        normal=self.outputs.new('NodeSocketFloat', "Normal")
        normal.hide_value=True

    def draw_buttons(self, context, layout):

        column=layout.column()
        im=None
        for image in bpy.data.images:
            if image.name == self.image:
                im=image
        split = column.split(percentage=0.8,align=True)
        split.prop_search(self,"image",context.blend_data,"images",text="")
        split.operator("pov.imageopen",text="",icon="FILESEL")
        if im is not None:
            column.prop(im,"source",text="")
        column.prop(self,"map_type",text="")
        column.prop(self,"interpolate",text="")
        column.prop(self,"once",text="Once")

    def draw_buttons_ext(self, context, layout):

        column=layout.column()
        im=None
        for image in bpy.data.images:
            if image.name == self.image:
                im=image
        split = column.split(percentage=0.8,align=True)
        split.prop_search(self,"image",context.blend_data,"images",text="")
        split.operator("pov.imageopen",text="",icon="FILESEL")
        if im is not None:
            column.prop(im,"source",text="")
        column.prop(self,"map_type",text="")
        column.prop(self,"interpolate",text="")
        column.prop(self,"once",text="Once")

    def draw_label(self):
        return "Bump Map"

class PovrayImagePatternNode(Node, ObjectNodeTree):
    '''ImagePattern'''
    bl_idname = 'PovrayImagePatternNode'
    bl_label = 'Image pattern'
    bl_icon = 'SOUND'

    map_type = bpy.props.EnumProperty(
            name="Map type",
            description="",
            items=(
                ('uv_mapping', "UV", ""),
                ('0', "Planar", "Default planar mapping"),
                ('1', "Spherical", "Spherical mapping"),
                ('2', "Cylindrical", "Cylindrical mapping"),
                ('5', "Torroidal", "Torus or donut shaped mapping"),
            ),
            default='0')
    image = StringProperty(maxlen=1024) # , subtype="FILE_PATH"
    interpolate = EnumProperty(
            name="Interpolate",
            description="Adding the interpolate keyword can smooth the jagged look of a bitmap",
            items=(
                ('2', "Bilinear", "Gives bilinear interpolation"),
                ('4', "Normalized", "Gives normalized distance"),
            ),
            default='2')
    premultiplied = BoolProperty(default=False)
    once = BoolProperty(description="Not to repeat", default=False)
    use_alpha = BoolProperty(default=True)
    def init(self, context):

        gamma=self.inputs.new('PovraySocketFloat_000001_10', "Gamma")
        gamma.default_value=2.0

        self.outputs.new('PovraySocketPattern', "Pattern")

    def draw_buttons(self, context, layout):

        column=layout.column()
        im=None
        for image in bpy.data.images:
            if image.name == self.image:
                im=image
        split = column.split(percentage=0.8,align=True)
        split.prop_search(self,"image",context.blend_data,"images",text="")
        split.operator("pov.imageopen",text="",icon="FILESEL")
        if im is not None:
            column.prop(im,"source",text="")
        column.prop(self,"map_type",text="")
        column.prop(self,"interpolate",text="")
        row=column.row()
        row.prop(self,"premultiplied",text="Premul")
        row.prop(self,"once",text="Once")
        column.prop(self,"use_alpha",text="Use alpha")

    def draw_buttons_ext(self, context, layout):

        column=layout.column()
        im=None
        for image in bpy.data.images:
            if image.name == self.image:
                im=image
        split = column.split(percentage=0.8,align=True)
        split.prop_search(self,"image",context.blend_data,"images",text="")
        split.operator("pov.imageopen",text="",icon="FILESEL")
        if im is not None:
            column.prop(im,"source",text="")
        column.prop(self,"map_type",text="")
        column.prop(self,"interpolate",text="")
        row=column.row()
        row.prop(self,"premultiplied",text="Premul")
        row.prop(self,"once",text="Once")

    def draw_label(self):
        return "Image pattern"

class ShaderPatternNode(Node, ObjectNodeTree):
    '''Pattern'''
    bl_idname = 'ShaderPatternNode'
    bl_label = 'Other patterns'

    pattern = EnumProperty(
            name="Pattern",
            description="Agate, Crackle, Gradient, Pavement, Spiral, Tiling",
            items=(('agate', "Agate", ""),('crackle', "Crackle", ""),('gradient', "Gradient", ""),
                   ('pavement', "Pavement", ""),
                   ('spiral1', "Spiral 1", ""),
                   ('spiral2', "Spiral 2", ""),
                   ('tiling', "Tiling", "")),
            default='agate')

    agate_turb = FloatProperty(
            name="Agate turb",
            description="Agate turbulence",
            min=0.0, max=100.0, default=0.5)

    crackle_form_x = FloatProperty(
            name="X",
            description="Form vector X",
            min=-150.0, max=150.0, default=-1)

    crackle_form_y = FloatProperty(
            name="Y",
            description="Form vector Y",
            min=-150.0, max=150.0, default=1)

    crackle_form_z = FloatProperty(
            name="Z",
            description="Form vector Z",
            min=-150.0, max=150.0, default=0)

    crackle_metric = FloatProperty(
            name="Metric",
            description="Crackle metric",
            min=0.0, max=150.0, default=1)

    crackle_solid = BoolProperty(
            name="Solid",
            description="Crackle solid",
            default=False)

    spiral_arms = FloatProperty(
            name="Number",
            description="",
            min=0.0, max=256.0, default=2.0)

    tiling_number = IntProperty(
            name="Number",
            description="",
            min=1, max=27, default=1)

    gradient_orient = EnumProperty(
            name="Orient",
            description="",
            items=(('x', "X", ""),
                   ('y', "Y", ""),
                   ('z', "Z", "")),
            default='x')

    def init(self, context):

        pat = self.outputs.new('PovraySocketPattern', "Pattern")

    def draw_buttons(self, context, layout):

        layout.prop(self, "pattern",text="")
        if self.pattern=='agate':
            layout.prop(self, "agate_turb")
        if self.pattern=='crackle':
            layout.prop(self, "crackle_metric")
            layout.prop(self, "crackle_solid")
            layout.label("Form:")
            layout.prop(self, "crackle_form_x")
            layout.prop(self, "crackle_form_y")
            layout.prop(self, "crackle_form_z")
        if self.pattern in {"spiral1","spiral2"}:
            layout.prop(self, "spiral_arms")
        if self.pattern in {'tiling'}:
            layout.prop(self, "tiling_number")
        if self.pattern in {'gradient'}:
            layout.prop(self, "gradient_orient")
    def draw_buttons_ext(self, context, layout):
        pass

    def draw_label(self):
        return "Other patterns"

class ShaderTextureMapNode(Node, ObjectNodeTree):
    '''Texture Map'''
    bl_idname = 'ShaderTextureMapNode'
    bl_label = 'Texture map'

    brick_size_x = FloatProperty(
            name="X",
            description="",
            min=0.0000, max=1.0000, default=0.2500)

    brick_size_y = FloatProperty(
            name="Y",
            description="",
            min=0.0000, max=1.0000, default=0.0525)

    brick_size_z = FloatProperty(
            name="Z",
            description="",
            min=0.0000, max=1.0000, default=0.1250)

    brick_mortar = FloatProperty(
            name="Mortar",
            description="Mortar",
            min=0.000, max=1.500, default=0.01)

    def init(self, context):
        mat = bpy.context.object.active_material
        self.inputs.new('PovraySocketPattern', "")
        color = self.inputs.new('NodeSocketColor', "Color ramp")
        color.hide_value = True
        for i in range(0,4):
            transform=self.inputs.new('PovraySocketTransform', "Transform")
            transform.hide_value=True
        number = mat.pov.inputs_number
        for i in range(number):
            self.inputs.new('PovraySocketTexture', "%s"%i)


        self.outputs.new('PovraySocketTexture', "Texture")

    def draw_buttons(self, context, layout):

        if self.inputs[0].default_value =='brick':
            layout.prop(self, "brick_mortar")
            layout.label("Brick size:")
            layout.prop(self, "brick_size_x")
            layout.prop(self, "brick_size_y")
            layout.prop(self, "brick_size_z")

    def draw_buttons_ext(self, context, layout):

        if self.inputs[0].default_value =='brick':
            layout.prop(self, "brick_mortar")
            layout.label("Brick size:")
            layout.prop(self, "brick_size_x")
            layout.prop(self, "brick_size_y")
            layout.prop(self, "brick_size_z")

    def draw_label(self):
        return "Texture map"


class ShaderNormalMapNode(Node, ObjectNodeTree):
    '''Normal Map'''
    bl_idname = 'ShaderNormalMapNode'
    bl_label = 'Normal map'

    brick_size_x = FloatProperty(
            name="X",
            description="",
            min=0.0000, max=1.0000, default=0.2500)

    brick_size_y = FloatProperty(
            name="Y",
            description="",
            min=0.0000, max=1.0000, default=0.0525)

    brick_size_z = FloatProperty(
            name="Z",
            description="",
            min=0.0000, max=1.0000, default=0.1250)

    brick_mortar = FloatProperty(
            name="Mortar",
            description="Mortar",
            min=0.000, max=1.500, default=0.01)

    def init(self, context):
        self.inputs.new('PovraySocketPattern', "")
        normal = self.inputs.new('PovraySocketFloat_10', "Normal")
        slope = self.inputs.new('PovraySocketMap', "Slope map")
        for i in range(0,4):
            transform=self.inputs.new('PovraySocketTransform', "Transform")
            transform.hide_value=True
        self.outputs.new('PovraySocketNormal', "Normal")

    def draw_buttons(self, context, layout):
        #for i, inp in enumerate(self.inputs):

        if self.inputs[0].default_value =='brick':
            layout.prop(self, "brick_mortar")
            layout.label("Brick size:")
            layout.prop(self, "brick_size_x")
            layout.prop(self, "brick_size_y")
            layout.prop(self, "brick_size_z")

    def draw_buttons_ext(self, context, layout):

        if self.inputs[0].default_value =='brick':
            layout.prop(self, "brick_mortar")
            layout.label("Brick size:")
            layout.prop(self, "brick_size_x")
            layout.prop(self, "brick_size_y")
            layout.prop(self, "brick_size_z")

    def draw_label(self):
        return "Normal map"

class ShaderNormalMapEntryNode(Node, ObjectNodeTree):
    '''Normal Map Entry'''
    bl_idname = 'ShaderNormalMapEntryNode'
    bl_label = 'Normal map entry'

    def init(self, context):
        self.inputs.new('PovraySocketFloat_0_1', "Stop")
        self.inputs.new('PovraySocketFloat_0_1', "Gray")
    def draw_label(self):
        return "Normal map entry"

class IsoPropsNode(Node, CompositorNodeTree):
    '''ISO Props'''
    bl_idname = 'IsoPropsNode'
    bl_label = 'Iso'
    node_label = StringProperty(maxlen=1024)
    def init(self, context):
        ob = bpy.context.object
        self.node_label = ob.name
        textName = ob.pov.function_text
        if textName:
            text = bpy.data.texts[textName]
            for line in text.lines:
                split = line.body.split()
                if split[0] == "#declare":
                    socket = self.inputs.new('NodeSocketFloat', "%s"%split[1])
                    value = split[3].split(";")
                    value = value[0]
                    socket.default_value=float(value)
    def draw_label(self):
        return self.node_label

class PovrayFogNode(Node, CompositorNodeTree):
    '''Fog settings'''
    bl_idname = 'PovrayFogNode'
    bl_label = 'Fog'
    def init(self, context):
        color=self.inputs.new('NodeSocketColor', "Color")
        color.default_value=(0.7,0.7,0.7,0.25)
        self.inputs.new('PovraySocketFloat_0_1', "Filter")
        distance = self.inputs.new('NodeSocketInt', "Distance")
        distance.default_value=150
        self.inputs.new('NodeSocketBool', "Ground")
        fog_offset=self.inputs.new('NodeSocketFloat', "Offset")
        fog_alt=self.inputs.new('NodeSocketFloat', "Altitude")
        turb = self.inputs.new('NodeSocketVector', "Turbulence")
        turb_depth=self.inputs.new('PovraySocketFloat_0_10', "Depth")
        turb_depth.default_value=0.5
        octaves=self.inputs.new('PovraySocketInt_1_9', "Octaves")
        octaves.default_value=5
        lambdat=self.inputs.new('PovraySocketFloat_0_10', "Lambda")
        lambdat.default_value=1.25
        omega=self.inputs.new('PovraySocketFloat_0_10', "Omega")
        omega.default_value=0.35
        translate = self.inputs.new('NodeSocketVector', "Translate")
        rotate = self.inputs.new('NodeSocketVector', "Rotate")
        scale = self.inputs.new('NodeSocketVector', "Scale")
        scale.default_value=(1,1,1)
    def draw_label(self):
        return "Fog"

class PovraySlopeNode(Node, TextureNodeTree):
    '''Output'''
    bl_idname = 'PovraySlopeNode'
    bl_label = 'Slope Map'

    def init(self, context):
        self.use_custom_color = True
        self.color = (0,0.2,0)
        slope = self.inputs.new('PovraySocketSlope', "0")
        slope = self.inputs.new('PovraySocketSlope', "1")
        slopemap = self.outputs.new('PovraySocketMap', "Slope map")
        output.hide_value = True
    def draw_buttons(self, context, layout):

        layout.operator("pov.nodeinputadd")
        row = layout.row()
        row.label('Value')
        row.label('Height')
        row.label('Slope')

    def draw_buttons_ext(self, context, layout):

        layout.operator("pov.nodeinputadd")
        row = layout.row()
        row.label('Value')
        row.label('Height')
        row.label('Slope')

    def draw_label(self):
        return "Slope Map"

######################################## Texture nodes ###############################
class TextureOutputNode(Node, TextureNodeTree):
    '''Output'''
    bl_idname = 'TextureOutputNode'
    bl_label = 'Color Map'

    def init(self, context):
        tex = bpy.context.object.active_material.active_texture
        num_sockets = int(tex.pov.density_lines/32)
        for i in range(num_sockets):
            color = self.inputs.new('NodeSocketColor', "%s"%i)
            color.hide_value = True

    def draw_buttons(self, context, layout):

        layout.label("Color Ramps:")

    def draw_label(self):
        return "Color Map"


##################################################################################
#################################Operators########################################
##################################################################################


class NODE_OT_iso_add(bpy.types.Operator):
    bl_idname = "pov.nodeisoadd"
    bl_label = "Create iso props"

    def execute(self, context):
        ob = bpy.context.object
        if bpy.context.scene.use_nodes == False:
            bpy.context.scene.use_nodes = True
        tree = bpy.context.scene.node_tree
        for node in tree.nodes:
            if node.bl_idname == "IsoPropsNode" and node.label == ob.name:
                tree.nodes.remove(node)
        isonode = tree.nodes.new('IsoPropsNode')
        isonode.location = (0,0)
        isonode.label = ob.name
        return {'FINISHED'}

class NODE_OT_map_create(bpy.types.Operator):
    bl_idname = "node.map_create"
    bl_label = "Create map"

    def execute(self, context):
        x = y = 0
        space = context.space_data
        tree = space.edit_tree
        for node in tree.nodes:
            if node.select == True:
                x,y = node.location
            node.select=False
        tmap = tree.nodes.new('ShaderTextureMapNode')
        tmap.location = (x - 200,y)
        return {'FINISHED'}

    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)

    def draw(self, context):
        layout = self.layout
        mat = context.object.active_material
        layout.prop(mat.pov,"inputs_number")

class NODE_OT_povray_node_texture_map_add(bpy.types.Operator):
    bl_idname = "pov.nodetexmapadd"
    bl_label = "Texture map"

    def execute(self, context):
        tree=bpy.context.object.active_material.node_tree
        tmap = tree.nodes.active
        bpy.context.object.active_material.node_tree.nodes.active=tmap
        el=tmap.color_ramp.elements.new(0.5)
        for el in tmap.color_ramp.elements:
            el.color=(0,0,0,1)
        for inp in tmap.inputs:
            tmap.inputs.remove(inp)
        for outp in tmap.outputs:
            tmap.outputs.remove(outp)
        pattern=tmap.inputs.new('NodeSocketVector', "Pattern")
        pattern.hide_value=True
        for i in range(0,3):
            tmap.inputs.new('NodeSocketColor', "Shader")
        tmap.outputs.new('NodeSocketShader', "BSDF")
        tmap.label="Texture Map"
        return {'FINISHED'}


class NODE_OT_povray_node_output_add(bpy.types.Operator):
    bl_idname = "pov.nodeoutputadd"
    bl_label = "Output"

    def execute(self, context):
        tree=bpy.context.object.active_material.node_tree
        tmap = tree.nodes.new('ShaderNodeOutputMaterial')
        bpy.context.object.active_material.node_tree.nodes.active=tmap
        for inp in tmap.inputs:
            tmap.inputs.remove(inp)
        tmap.inputs.new('NodeSocketShader', "Surface")
        tmap.label="Output"
        return {'FINISHED'}

class NODE_OT_povray_node_layered_add(bpy.types.Operator):
    bl_idname = "pov.nodelayeredadd"
    bl_label = "Layered material"

    def execute(self, context):
        tree=bpy.context.object.active_material.node_tree
        tmap = tree.nodes.new('ShaderNodeAddShader')
        bpy.context.object.active_material.node_tree.nodes.active=tmap
        tmap.label="Layered material"
        return {'FINISHED'}

class NODE_OT_povray_input_add(bpy.types.Operator):
    bl_idname = "pov.nodeinputadd"
    bl_label = "Add entry"

    def execute(self, context):
        node=bpy.context.object.active_material.node_tree.nodes.active
        if node.type in {'VALTORGB'}:
            number=1
            for inp in node.inputs:
                if inp.type=='SHADER':
                    number+=1
            node.inputs.new('NodeSocketShader', "%s"%number)
            els=node.color_ramp.elements
            pos1=els[len(els)-1].position
            pos2=els[len(els)-2].position
            pos=(pos1-pos2)/2+pos2
            el=els.new(pos)

        if node.bl_idname == 'PovraySlopeNode':
            number=len(node.inputs)
            node.inputs.new('PovraySocketSlope', "%s"%number)


        return {'FINISHED'}

class NODE_OT_povray_input_remove(bpy.types.Operator):
    bl_idname = "pov.nodeinputremove"
    bl_label = "Remove input"

    def execute(self, context):
        node=bpy.context.object.active_material.node_tree.nodes.active
        if node.type in {'VALTORGB','ADD_SHADER'}:
            number=len(node.inputs)-1
            if number > 5:
                inp=node.inputs[number]
                node.inputs.remove(inp)
                if node.type in {'VALTORGB'}:
                    els=node.color_ramp.elements
                    number=len(els)-2
                    el=els[number]
                    els.remove(el)
        return {'FINISHED'}

class NODE_OT_povray_image_open(bpy.types.Operator):
    bl_idname = "pov.imageopen"
    bl_label = "Open"

    filepath = StringProperty(
            name="File Path",
            description="Open image",
            maxlen=1024,
            subtype='FILE_PATH',
            )

    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

    def execute(self, context):
        im=bpy.data.images.load(self.filepath)
        node=context.object.active_material.node_tree.nodes.active
        node.image = im.name
        return {'FINISHED'}


# class TEXTURE_OT_povray_open_image(bpy.types.Operator):
    # bl_idname = "pov.openimage"
    # bl_label = "Open Image"

    # filepath = StringProperty(
            # name="File Path",
            # description="Open image",
            # maxlen=1024,
            # subtype='FILE_PATH',
            # )

    # def invoke(self, context, event):
        # context.window_manager.fileselect_add(self)
        # return {'RUNNING_MODAL'}

    # def execute(self, context):
        # im=bpy.data.images.load(self.filepath)
        # tex = context.texture
        # tex.pov.image = im.name
        # scene = context.scene
        # scene.update()
        # return {'FINISHED'}

class PovrayPatternNode(bpy.types.Operator):
    bl_idname = "pov.patternnode"
    bl_label  = "Pattern"

    add=True

    def execute(self, context):
        space = context.space_data
        tree = space.edit_tree
        for node in tree.nodes:
            node.select=False
        if self.add==True:
            tmap = tree.nodes.new('ShaderNodeValToRGB')
            tmap.label="Pattern"
            for inp in tmap.inputs:
                tmap.inputs.remove(inp)
            for outp in tmap.outputs:
                tmap.outputs.remove(outp)
            pattern = tmap.inputs.new('PovraySocketPattern', "Pattern")
            pattern.hide_value = True
            mapping=tmap.inputs.new('NodeSocketVector', "Mapping")
            mapping.hide_value=True
            transform=tmap.inputs.new('NodeSocketVector', "Transform")
            transform.hide_value=True
            modifier=tmap.inputs.new('NodeSocketVector', "Modifier")
            modifier.hide_value=True
            for i in range(0,2):
                tmap.inputs.new('NodeSocketShader', "%s"%(i+1))
            tmap.outputs.new('NodeSocketShader', "Material")
            tmap.outputs.new('NodeSocketColor', "Color")
            tree.nodes.active=tmap
            self.add=False
        aNode=tree.nodes.active
        aNode.select=True
        v2d = context.region.view2d
        x, y = v2d.region_to_view(self.x, self.y)
        aNode.location = (x, y)

    def modal(self, context, event):
        if event.type == 'MOUSEMOVE':
            self.x = event.mouse_region_x
            self.y = event.mouse_region_y
            self.execute(context)
            return {'RUNNING_MODAL'}
        elif event.type == 'LEFTMOUSE':
            return {'FINISHED'}
        elif event.type in ('RIGHTMOUSE', 'ESC'):
            return {'CANCELLED'}

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

class UpdatePreviewMaterial(bpy.types.Operator):
    '''Operator update preview material'''
    bl_idname = "node.updatepreview"
    bl_label = "Update preview"

    def execute(self, context):
        scene=context.scene
        ob=context.object
        for obj in scene.objects:
            if obj != ob:
                scene.objects.active=obj
                break
        scene.objects.active=ob

    def modal(self, context, event):
        if event.type == 'RIGHTMOUSE':
            self.execute(context)
            return {'FINISHED'}
        return {'PASS_THROUGH'}

    def invoke(self, context, event):
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

class UpdatePreviewKey(bpy.types.Operator):
    '''Operator update preview keymap'''
    bl_idname = "wm.updatepreviewkey"
    bl_label = "Activate RMB"
    @classmethod
    def poll(cls, context):
        conf = context.window_manager.keyconfigs.active
        mapstr = "Node Editor"
        map = conf.keymaps[mapstr]
        try:
            map.keymap_items["node.updatepreview"]
            return False
        except:
            return True

    def execute(self, context):
        conf = context.window_manager.keyconfigs.active
        mapstr = "Node Editor"
        map = conf.keymaps[mapstr]
        map.keymap_items.new("node.updatepreview",type='RIGHTMOUSE',value="PRESS")
        return {'FINISHED'}




# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8-80 compliant>

"""
Give access to blender data and utility functions.
"""

__all__ = (
    "app",
    "context",
    "data",
    "ops",
    "path",
    "props",
    "types",
    "utils",
)


# internal blender C module
from _bpy import types, props, app, data, context

# python modules
from . import utils, path

# fake operator module
from .ops import ops_fake_module as ops


def main():
    import sys

    # Possibly temp. addons path
    from os.path import join, dirname
    sys.path.extend([
        join(dirname(dirname(dirname(__file__))), "addons", "modules"),
        join(utils.user_resource('SCRIPTS'), "addons", "modules"),
    ])

    # fake module to allow:
    #   from bpy.types import Panel
    sys.modules.update({
        "bpy.app": app,
        "bpy.app.handlers": app.handlers,
        "bpy.app.translations": app.translations,
        "bpy.types": types,
    })

    # Initializes Python classes.
    # (good place to run a profiler or trace).
    utils.load_scripts()


main()

del main



# space_view_3d_display_tools.py Copyright (C) 2014, Jordi Vall-llovera
# Multiple display tools for fast navigate/interact with the viewport

# ##### BEGIN GPL LICENSE BLOCK #####
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENCE BLOCK #####

"""
Additional links:
    Author Site: http://www.jordiart.com
"""

import bpy
from bpy.types import Operator
from bpy.props import BoolProperty
from collections import defaultdict


# Fast Navigate toggle function
def trigger_fast_navigate(trigger):
    scene = bpy.context.scene.display_tools
    scene.FastNavigateStop = False
    trigger = not trigger


# Control how to display particles during fast navigate
def display_particles(mode, dis_particles):
    scene = bpy.context.scene.display_tools

    for particles in bpy.data.particles:
        if scene.ShowParticles is False:
            particles.draw_method = 'NONE'
        else:
            if particles.type == 'EMITTER':
                particles.draw_method = 'DOT'
                particles.draw_percentage = 100
            else:
                particles.draw_method = 'RENDER'
                particles.draw_percentage = dis_particles

    return dis_particles


# Fast Navigate operator
class FastNavigate(Operator):
    bl_idname = "view3d.fast_navigate_operator"
    bl_label = "Fast Navigate"
    bl_description = ("Limit the objects drawing in the 3D view for faster navigation\n"
                      "Runs in modal mode until Stop is pressed or Esc, Return, Space")

    trigger = BoolProperty(default=False)
    mode = BoolProperty(default=False)
    screen_width = [0, 0]
    store_fail = False
    store_init_particles = {}
    store_viewport_shade = None
    check_particles_draw = False
    fast_particles_draw = 0

    def modal(self, context, event):
        context.area.tag_redraw()
        scene = context.scene.display_tools

        if scene.FastNavigateStop is True:
            self.execute(context)
            return {'FINISHED'}

        if context.area.type != 'VIEW_3D':
            self.execute(context)
            return {'FINISHED'}

        if scene.EditActive is True:
            self.fast_navigate_stuff(context, event)
            return {'PASS_THROUGH'}
        else:
            obj = context.active_object
            if obj:
                if obj.mode != 'EDIT':
                    self.fast_navigate_stuff(context, event)
                    return {'PASS_THROUGH'}
                else:
                    return {'PASS_THROUGH'}
            else:
                self.fast_navigate_stuff(context, event)
                return {'PASS_THROUGH'}

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        if context.area.type != 'VIEW_3D':
            self.report({'WARNING'},
                        "Fast Navigate: View3D not found. Operation Cancelled")
            return {'CANCELLED'}

        self.store_init_particles = defaultdict(list)
        self.store_fail = False

        context.window_manager.modal_handler_add(self)
        trigger_fast_navigate(self.trigger)
        scene = context.scene.display_tools
        scene.DelayTime = scene.DelayTimeGlobal
        self.get_screen_size(context, scene)
        self.start_settings_store(context, store=False)
        self.check_particles_draw = True

        return {'RUNNING_MODAL'}

    def execute(self, context):
        scene = context.scene.display_tools
        scene.FastNavigateStop = True

        self.start_settings_store(context, store=True)

        mess, mess_txt = ('WARNING', "Fast Navigate: Some Settings could not be restored") if \
                    self.store_fail else ('INFO', "Exited the Fast Navigate mode")

        self.report({mess}, mess_txt)

        return {'FINISHED'}

    @staticmethod
    def calc_delay(scene):
        if scene.Delay is True:
            if scene.DelayTime < scene.DelayTimeGlobal:
                scene.DelayTime += 1

    def start_settings_store(self, context, store=False):
        try:
            view = context.space_data
            shade = view.viewport_shade if view.type == 'VIEW_3D' else None
            if store is False:
                if not shade:
                    self.store_fail = True
                else:
                    self.store_viewport_shade = shade
                for particle in bpy.data.particles:
                    self.store_init_particles[particle.name] = \
                        [particle.draw_method, particle.draw_percentage]
            else:
                if not shade:
                    self.store_fail = True
                else:
                    shade = self.store_viewport_shade or 'SOLID'
                for particle in bpy.data.particles:
                    particle.draw_method = self.store_init_particles[particle.name][0]
                    particle.draw_percentage = self.store_init_particles[particle.name][1]
        except:
            self.store_fail = True

    def get_screen_size(self, context, scene):
        if context.area.type == 'VIEW_3D':
            coord_x = context.area.x + scene.ScreenStart
            coord_max_x = context.area.width - scene.ScreenEnd
            self.screen_width = [coord_x, coord_max_x]

    # Do repetitive fast navigate related stuff
    def fast_navigate_stuff(self, context, event):
        scene = context.scene.display_tools
        view = context.space_data

        if scene.FastNavigateStop is True:
            return {'FINISHED'}

        if context.area.type != 'VIEW_3D':
            scene.FastNavigateStop = True
            return {'FINISHED'}

        if event.type in {'ESC', 'RET', 'SPACE'}:
            scene.FastNavigateStop = True
            return {'CANCELLED'}

        # limit the active area
        if event.mouse_x not in range(self.screen_width[0], self.screen_width[1]):
            return {'PASS_THROUGH'}

        # fast navigate while orbit/panning
        if event.type == 'MIDDLEMOUSE':
            self.calc_delay(scene)
            view.viewport_shade = scene.FastMode
            self.mode = False

        # fast navigate while transform operations
        if event.type in {'G', 'R', 'S'}:
            self.calc_delay(scene)
            view.viewport_shade = scene.FastMode
            self.mode = False

        # fast navigate while menu popups or duplicates
        if event.type in {'W', 'D', 'L', 'U', 'I', 'M', 'A', 'B'}:
            self.calc_delay(scene)
            view.viewport_shade = scene.FastMode
            self.mode = False

        # fast navigate while numpad navigation
        if event.type in {'NUMPAD_PERIOD', 'NUMPAD_1', 'NUMPAD_2', 'NUMPAD_3',
                          'NUMPAD_4', 'NUMPAD_5', 'NUMPAD_6', 'NUMPAD_7',
                          'NUMPAD_8', 'NUMPAD_9'}:
            self.calc_delay(scene)
            view.viewport_shade = scene.FastMode
            self.mode = False

        # fast navigate while zooming with mousewheel too
        if event.type in {'WHEELUPMOUSE', 'WHEELDOWNMOUSE'}:
            scene.DelayTime = scene.DelayTimeGlobal
            view.viewport_shade = scene.FastMode
            self.mode = False

        if event.type == 'MOUSEMOVE':
            if scene.Delay is True:
                if scene.DelayTime == 0:
                    view.viewport_shade = scene.OriginalMode
                    scene.DelayTime = scene.DelayTimeGlobal
                    self.mode = True
            else:
                view.viewport_shade = scene.OriginalMode
                self.mode = True

        if scene.Delay is True:
            scene.DelayTime -= 1
            if scene.DelayTime == 0:
                view.viewport_shade = scene.OriginalMode
                scene.DelayTime = scene.DelayTimeGlobal
                self.mode = True

        # update particles draw (only update call when the fast draw or the percentage are changed)
        self.check_particles_draw = not self.mode
        dis_particles = scene.InitialParticles if self.mode else scene.ParticlesPercentageDisplay

        if self.check_particles_draw is True:
            self.fast_particles_draw = display_particles(self.mode, dis_particles)
        else:
            if self.fast_particles_draw != dis_particles:
                self.fast_particles_draw = display_particles(self.mode, dis_particles)


# Fast Navigate Stop
def fast_navigate_stop(context):
    scene = context.scene.display_tools
    scene.FastNavigateStop = True


# Fast Navigate Stop Operator
class FastNavigateStop(Operator):
    bl_idname = "view3d.fast_navigate_stop"
    bl_label = "Stop"
    bl_description = "Stop Fast Navigate"

    def execute(self, context):
        fast_navigate_stop(context)

        return {'FINISHED'}


# Register
def register():
    bpy.utils.register_module(__name__)


def unregister():
    bpy.utils.unregister_module(__name__)


if __name__ == "__main__":
    register()



# GPL Original by Fourmadmen

import bpy
from mathutils import (
        Vector,
        Quaternion,
        )
from math import pi
from bpy.props import (
        IntProperty,
        FloatProperty,
        )


# Create a new mesh (object) from verts/edges/faces.
# verts/edges/faces ... List of vertices/edges/faces for the
#                       new mesh (as used in from_pydata)
# name ... Name of the new mesh (& object)

def create_mesh_object(context, verts, edges, faces, name):

    # Create new mesh
    mesh = bpy.data.meshes.new(name)

    # Make a mesh from a list of verts/edges/faces.
    mesh.from_pydata(verts, edges, faces)

    # Update mesh geometry after adding stuff.
    mesh.update()

    from bpy_extras import object_utils
    return object_utils.object_data_add(context, mesh, operator=None)


# A very simple "bridge" tool.

def createFaces(vertIdx1, vertIdx2, closed=False, flipped=False):
    faces = []

    if not vertIdx1 or not vertIdx2:
        return None

    if len(vertIdx1) < 2 and len(vertIdx2) < 2:
        return None

    fan = False
    if (len(vertIdx1) != len(vertIdx2)):
        if (len(vertIdx1) == 1 and len(vertIdx2) > 1):
            fan = True
        else:
            return None

    total = len(vertIdx2)

    if closed:
        # Bridge the start with the end.
        if flipped:
            face = [
                vertIdx1[0],
                vertIdx2[0],
                vertIdx2[total - 1]]
            if not fan:
                face.append(vertIdx1[total - 1])
            faces.append(face)

        else:
            face = [vertIdx2[0], vertIdx1[0]]
            if not fan:
                face.append(vertIdx1[total - 1])
            face.append(vertIdx2[total - 1])
            faces.append(face)

    # Bridge the rest of the faces.
    for num in range(total - 1):
        if flipped:
            if fan:
                face = [vertIdx2[num], vertIdx1[0], vertIdx2[num + 1]]
            else:
                face = [vertIdx2[num], vertIdx1[num],
                        vertIdx1[num + 1], vertIdx2[num + 1]]
            faces.append(face)
        else:
            if fan:
                face = [vertIdx1[0], vertIdx2[num], vertIdx2[num + 1]]
            else:
                face = [vertIdx1[num], vertIdx2[num],
                        vertIdx2[num + 1], vertIdx1[num + 1]]
            faces.append(face)

    return faces


# @todo Clean up vertex&face creation process a bit.

def add_star(points, outer_radius, inner_radius, height):
    PI_2 = pi * 2
    z_axis = (0, 0, 1)

    verts = []
    faces = []

    segments = points * 2

    half_height = height / 2.0

    vert_idx_top = len(verts)
    verts.append(Vector((0.0, 0.0, half_height)))

    vert_idx_bottom = len(verts)
    verts.append(Vector((0.0, 0.0, -half_height)))

    edgeloop_top = []
    edgeloop_bottom = []

    for index in range(segments):
        quat = Quaternion(z_axis, (index / segments) * PI_2)

        if index % 2:
            # Uneven
            radius = outer_radius
        else:
            # Even
            radius = inner_radius

        edgeloop_top.append(len(verts))
        vec = quat * Vector((radius, 0, half_height))
        verts.append(vec)

        edgeloop_bottom.append(len(verts))
        vec = quat * Vector((radius, 0, -half_height))
        verts.append(vec)

    faces_top = createFaces([vert_idx_top], edgeloop_top, closed=True)
    faces_outside = createFaces(edgeloop_top, edgeloop_bottom, closed=True)
    faces_bottom = createFaces([vert_idx_bottom], edgeloop_bottom,
        flipped=True, closed=True)

    faces.extend(faces_top)
    faces.extend(faces_outside)
    faces.extend(faces_bottom)

    return verts, faces


class AddStar(bpy.types.Operator):
    bl_idname = "mesh.primitive_star_add"
    bl_label = "Simple Star"
    bl_description = "Construct a star mesh"
    bl_options = {'REGISTER', 'UNDO', 'PRESET'}

    points = IntProperty(
        name="Points",
        description="Number of points for the star",
        min=2,
        max=256,
        default=5
        )
    outer_radius = FloatProperty(
        name="Outer Radius",
        description="Outer radius of the star",
        min=0.01,
        max=9999.0,
        default=1.0
        )
    innter_radius = FloatProperty(
        name="Inner Radius",
        description="Inner radius of the star",
        min=0.01,
        max=9999.0,
        default=0.5
        )
    height = FloatProperty(name="Height",
        description="Height of the star",
        min=0.01,
        max=9999.0,
        default=0.5
        )

    def execute(self, context):

        verts, faces = add_star(
            self.points,
            self.outer_radius,
            self.innter_radius,
            self.height
            )

        obj = create_mesh_object(context, verts, [], faces, "Star")

        return {'FINISHED'}



# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# --------------------------------- UV to MESH ------------------------------- #
# -------------------------------- version 0.1.1 ----------------------------- #
#                                                                              #
# Create a new Mesh based on active UV                                         #
#                                                                              #
#                        (c)   Alessandro Zomparelli                           #
#                                    (2017)                                    #
#                                                                              #
# http://www.co-de-it.com/                                                     #
#                                                                              #
# ############################################################################ #

bl_info = {
    "name": "UV to Mesh",
    "author": "Alessandro Zomparelli (Co-de-iT)",
    "version": (0, 1, 1),
    "blender": (2, 7, 9),
    "location": "",
    "description": "Create a new Mesh based on active UV",
    "warning": "",
    "wiki_url": "",
    "category": "Mesh"}


import bpy
import math
from bpy.types import Operator
from bpy.props import BoolProperty
from mathutils import Vector


class uv_to_mesh(Operator):
    bl_idname = "object.uv_to_mesh"
    bl_label = "UV to Mesh"
    bl_description = ("Create a new Mesh based on active UV")
    bl_options = {'REGISTER', 'UNDO'}

    apply_modifiers = BoolProperty(
            name="Apply Modifiers",
            default=False,
            description="Apply object's modifiers"
            )
    vertex_groups = BoolProperty(
            name="Keep Vertex Groups",
            default=False,
            description="Transfer all the Vertex Groups"
            )
    materials = BoolProperty(
            name="Keep Materials",
            default=True,
            description="Transfer all the Materials"
            )
    auto_scale = BoolProperty(
            name="Resize",
            default=True,
            description="Scale the new object in order to preserve the average surface area"
            )

    def execute(self, context):
        bpy.ops.object.mode_set(mode='OBJECT')
        for o in bpy.data.objects:
            o.select = False
        bpy.context.object.select = True

        if self.apply_modifiers:
            bpy.ops.object.duplicate_move()
            bpy.ops.object.convert(target='MESH')
        ob0 = bpy.context.object

        me0 = ob0.to_mesh(bpy.context.scene,
                    apply_modifiers=self.apply_modifiers, settings='PREVIEW')
        area = 0

        verts = []
        faces = []
        face_materials = []
        for face in me0.polygons:
            area += face.area
            uv_face = []
            store = False
            try:
                for loop in face.loop_indices:
                    uv = me0.uv_layers.active.data[loop].uv
                    if uv.x != 0 and uv.y != 0:
                        store = True
                    new_vert = Vector((uv.x, uv.y, 0))
                    verts.append(new_vert)
                    uv_face.append(loop)
                if store:
                    faces.append(uv_face)
                    face_materials.append(face.material_index)
            except:
                self.report({'ERROR'}, "Missing UV Map")

                return {'CANCELLED'}

        name = ob0.name + 'UV'
        # Create mesh and object
        me = bpy.data.meshes.new(name + 'Mesh')
        ob = bpy.data.objects.new(name, me)

        # Link object to scene and make active
        scn = bpy.context.scene
        scn.objects.link(ob)
        scn.objects.active = ob
        ob.select = True

        # Create mesh from given verts, faces.
        me.from_pydata(verts, [], faces)
        # Update mesh with new data
        me.update()
        if self.auto_scale:
            new_area = 0
            for p in me.polygons:
                new_area += p.area
            if new_area == 0:
                self.report({'ERROR'}, "Impossible to generate mesh from UV")

                return {'CANCELLED'}

        # VERTEX GROUPS
        if self.vertex_groups:
            try:
                for group in ob0.vertex_groups:
                    index = group.index
                    ob.vertex_groups.new(group.name)
                    for p in me0.polygons:
                        for vert, loop in zip(p.vertices, p.loop_indices):
                            ob.vertex_groups[index].add([loop], group.weight(vert), "ADD")
            except:
                pass

        ob0.select = False
        if self.auto_scale:
            scaleFactor = math.pow(area / new_area, 1 / 2)
            ob.scale = Vector((scaleFactor, scaleFactor, scaleFactor))

        bpy.ops.object.mode_set(mode='EDIT', toggle=False)
        bpy.ops.mesh.remove_doubles(threshold=1e-06)
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

        # MATERIALS
        if self.materials:
            try:
                # assign old material
                uv_materials = [slot.material for slot in ob0.material_slots]
                for i in range(len(uv_materials)):
                    bpy.ops.object.material_slot_add()
                    bpy.context.object.material_slots[i].material = uv_materials[i]
                for i in range(len(ob.data.polygons)):
                    ob.data.polygons[i].material_index = face_materials[i]
            except:
                pass

        if self.apply_modifiers:
            bpy.ops.object.mode_set(mode='OBJECT')
            ob.select = False
            ob0.select = True
            bpy.ops.object.delete(use_global=False)
            ob.select = True
            bpy.context.scene.objects.active = ob

        return {'FINISHED'}


def register():
    bpy.utils.register_class(uv_to_mesh)


def unregister():
    bpy.utils.unregister_class(uv_to_mesh)


if __name__ == "__main__":
    register()



#!/usr/bin/env python3

# ***** BEGIN GPL LICENSE BLOCK *****
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Contributor(s): Sergey Sharybin
#
# #**** END GPL LICENSE BLOCK #****

# <pep8 compliant>


import unittest


def sliceCommandLineArguments():
    """
    Slice command line arguments by -- argument.
    """

    import sys

    try:
        double_shasl_index = sys.argv.index("--")
    except ValueError:
        unittest_args = sys.argv[:]
        parser_args = []
    else:
        unittest_args = sys.argv[:double_shasl_index]
        parser_args = sys.argv[double_shasl_index + 1:]

    return unittest_args, parser_args


def parseArguments():
    import argparse

    # Construct argument parser.
    parser = argparse.ArgumentParser(description="Static binary checker")
    parser.add_argument('directory', help='Directories to check')
    # Parse arguments which are not handled by unit testing framework.
    unittest_args, parser_args = sliceCommandLineArguments()
    args = parser.parse_args(args=parser_args)
    # TODO(sergey): Run some checks here?
    return args


def runScriptInBlender(blender_directory, script):
    """
    Run given script inside Blender and check non-zero exit code
    """

    import os
    import subprocess

    blender = os.path.join(blender_directory, "blender")
    python = os.path.join(os.path.dirname(__file__), "scripts", script) + ".py"

    command = (blender,
               "-b",
               "--factory-startup",
               "--python-exit-code", "1",
               "--python", python)

    process = subprocess.Popen(command,
                               shell=False,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.STDOUT)
    output, error = process.communicate()
    return process.returncode == 0


class SceiptUnitTesting(unittest.TestCase):
    def checkScript(self, script):
        # Parse arguments which are not handled by unit testing framework.
        args = parseArguments()
        # Perform actual test,
        self.assertTrue(runScriptInBlender(args.directory, script),
                        "Failed to run script {}" . format(script))



# ##### BEGIN GPL LICENSE BLOCK #####
#
# This program is free software; you may redistribute it, and/or
# modify it, under the terms of the GNU General Public License
# as published by the Free Software Foundation - either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, write to:
#
#   the Free Software Foundation Inc.
#   51 Franklin Street, Fifth Floor
#   Boston, MA 02110-1301, USA
#
# or go online at: http://www.gnu.org/licenses/ to view license options.
#
# ##### END GPL LICENCE BLOCK #####

# authors: dudecon, jambay

# This module contains the UI definition, display,
# and processing (create mesh) functions.
# The routines to generate the vertices for the wall
# are found in the "Blocks" module.


import bpy
from bpy.types import Operator
from bpy.props import (
        BoolProperty,
        FloatProperty,
        )
from .Blocks import (
        NOTZERO, PI,
        dims,
        settings,
        shelfSpecs,
        stepSpecs,
        createWall,
        radialized,
        slope,
        openingSpecs,
        bigBlock,
        shelfExt,
        stepMod,
        stepLeft,
        shelfBack,
        stepOnly,
        stepBack,
        )


class add_mesh_wallb(Operator):
    bl_idname = "mesh.wall_add"
    bl_label = "Add a Masonry Wall"
    bl_description = "Create a block (masonry) wall mesh"
    bl_options = {'REGISTER', 'UNDO'}

    # UI items - API for properties - User accessable variables...
    # not all options are via UI, and some operations just don't work yet

    # only create object when True
    # False allows modifying several parameters without creating object
    ConstructTog = BoolProperty(
            name="Construct",
            description="Generate the object",
            default=True
            )
    # need to modify so radial makes a tower (normal);
    # want "flat" setting to make disk (alternate)
    # make the wall circular - if not sloped it's a flat disc
    RadialTog = BoolProperty(
            name="Radial",
            description="Make masonry radial",
            default=False
            )
    # curve the wall - if radial creates dome.
    SlopeTog = BoolProperty(
            name="Curved",
            description="Make masonry sloped, or curved",
            default=False
            )
    # need to review defaults and limits for all of these UI objects

    # wall area/size
    WallStart = FloatProperty(
            name="Start",
            description="Left side, or start angle",
            default=-10.0,
            min=-100, max=100.0
            )
    WallEnd = FloatProperty(
            name="End",
            description="Right side, or end angle",
            default=10.0,
            min=0.0, max=100.0
            )
    WallBottom = FloatProperty(
            name="Bottom",
            description="Lower height or radius",
            default=0.0,
            min=-100, max=100
            )
    WallTop = FloatProperty(
            name="Top",
            description="Upper height or radius",
            default=15.0,
            min=0.0, max=100.0
            )
    EdgeOffset = FloatProperty(
            name="Edging",
            description="Block staggering on wall sides",
            default=0.6, min=0.0, max=100.0
            )
    # block sizing
    Width = FloatProperty(
            name="Width",
            description="Average width of each block",
            default=1.5,
            min=0.01, max=100.0
            )
    WidthVariance = FloatProperty(
            name="Variance",
            description="Random variance of block width",
            default=0.5,
            min=0.0, max=100.0
            )
    WidthMinimum = FloatProperty(
            name="Minimum",
            description="Absolute minimum block width",
            default=0.5,
            min=0.01, max=100.0
            )
    Height = FloatProperty(
            name="Height",
            description="Average Height of each block",
            default=0.7,
            min=0.01, max=100.0
            )
    HeightVariance = FloatProperty(
            name="Variance",
            description="Random variance of block Height",
            default=0.3,
            min=0.0, max=100.0
            )
    HeightMinimum = FloatProperty(
            name="Minimum",
            description="Absolute minimum block Height",
            default=0.25,
            min=0.01, max=100.0
            )
    Depth = FloatProperty(
            name="Depth",
            description="Average Depth of each block",
            default=2.0,
            min=0.01, max=100.0
            )
    DepthVariance = FloatProperty(
            name="Variance",
            description="Random variance of block Depth",
            default=0.1,
            min=0.0, max=100.0
            )
    DepthMinimum = FloatProperty(
            name="Minimum",
            description="Absolute minimum block Depth",
            default=1.0,
            min=0.01, max=100.0
            )
    MergeBlock = BoolProperty(
            name="Merge Blocks",
            description="Make big blocks (merge closely adjoining blocks)",
            default=False
            )
    # edging for blocks
    Grout = FloatProperty(
            name="Thickness",
            description="Distance between blocks",
            default=0.1,
            min=-10.0, max=10.0
            )
    GroutVariance = FloatProperty(
            name="Variance",
            description="Random variance of block Grout",
            default=0.03,
            min=0.0, max=100.0
            )
    GroutDepth = FloatProperty(
            name="Depth",
            description="Grout Depth from the face of the blocks",
            default=0.1,
            min=0.0001, max=10.0
            )
    GroutDepthVariance = FloatProperty(
            name="Variance",
            description="Random variance of block Grout Depth",
            default=0.03,
            min=0.0, max=100.0
            )
    GroutEdge = BoolProperty(
            name="Edging",
            description="Grout perimiter",
            default=False
            )
    # properties for openings
    Opening1Tog = BoolProperty(
            name="Opening(s)",
            description="Make windows or doors",
            default=True
            )
    Opening1Width = FloatProperty(
            name="Width",
            description="The Width of the first opening",
            default=2.5,
            min=0.01, max=100.0
            )
    Opening1Height = FloatProperty(
            name="Height",
            description="The Height of the first opening",
            default=3.5,
            min=0.01, max=100.0
            )
    Opening1X = FloatProperty(
            name="Indent",
            description="The x position or spacing of the first opening",
            default=5.0,
            min=-100, max=100.0
            )
    Opening1Z = FloatProperty(
            name="Bottom",
            description="The z position of the First opening",
            default=5.0,
            min=-100, max=100.0
            )
    Opening1Repeat = BoolProperty(
            name="Repeat",
            description="make multiple openings, with spacing X1",
            default=False
            )
    Opening1TopArchTog = BoolProperty(
            name="Top Arch",
            description="Add an arch to the top of the first opening",
            default=True
            )
    Opening1TopArch = FloatProperty(
            name="Curve",
            description="Height of the arch on the top of the opening",
            default=2.5,
            min=0.001, max=100.0
            )
    Opening1TopArchThickness = FloatProperty(
            name="Thickness",
            description="Thickness of the arch on the top of the opening",
            default=0.75,
            min=0.001, max=100.0
            )
    Opening1BtmArchTog = BoolProperty(
            name="Bottom Arch",
            description="Add an arch to the bottom of opening 1",
            default=False
            )
    Opening1BtmArch = FloatProperty(
            name="Curve",
            description="Height of the arch on the bottom of the opening",
            default=1.0,
            min=0.01, max=100.0
            )
    Opening1BtmArchThickness = FloatProperty(
            name="Thickness",
            description="Thickness of the arch on the bottom of the opening",
            default=0.5,
            min=0.01, max=100.0
            )
    Opening1Bevel = FloatProperty(
            name="Bevel",
            description="Angle block face",
            default=0.25,
            min=-10.0, max=10.0
            )
    # openings on top of wall
    CrenelTog = BoolProperty(
            name="Crenels",
            description="Make openings along top of wall",
            default=False
            )
    CrenelXP = FloatProperty(
            name="Width",
            description="Gap width in wall based the percentage of wall width",
            default=0.25,
            min=0.10, max=1.0,
            subtype="PERCENTAGE"
            )
    CrenelZP = FloatProperty(
            name="Height",
            description="Crenel Height as the percentage of wall height",
            default=0.10,
            min=0.10, max=1.0,
            subtype="PERCENTAGE"
            )
    # narrow openings in wall.
    # need to prevent overlap with arch openings - though inversion is an interesting effect.
    SlotTog = BoolProperty(
            name="Slots",
            description="Make narrow openings in wall",
            default=False
            )
    SlotRpt = BoolProperty(
            name="Repeat",
            description="Repeat slots along wall",
            default=False
            )
    SlotWdg = BoolProperty(
            name="Wedged (n/a)",
            description="Bevel edges of slots",
            default=False
            )
    SlotX = FloatProperty(
            name="Indent",
            description="The x position or spacing of slots",
            default=0.0, min=-100, max=100.0
            )
    SlotGap = FloatProperty(
            name="Opening",
            description="The opening size of slots",
            default=0.5, min=0.10, max=100.0
            )
    SlotV = BoolProperty(
            name="Vertical",
            description="Vertical slots",
            default=True
            )
    SlotVH = FloatProperty(
            name="Height",
            description="Height of vertical slot",
            default=3.5,
            min=0.10, max=100.0
            )
    SlotVBtm = FloatProperty(
            name="Bottom",
            description="Z position for slot",
            default=5.00,
            min=-100.0, max=100.0
            )
    SlotH = BoolProperty(
            name="Horizontal",
            description="Horizontal slots",
            default=False
            )
    SlotHW = FloatProperty(
            name="Width",
            description="Width of horizontal slot",
            default=2.5,
            min=0.10, max=100.0
            )
    # this should offset from VBtm... maybe make a % like crenels?
    SlotHBtm = FloatProperty(
            name="Bottom",
            description="Z position for horizontal slot",
            default=5.50,
            min=-100.0, max=100.0
            )
    # properties for shelf (extend blocks in area)
    ShelfTog = BoolProperty(
            name="Shelf",
            description="Add blocks in area by depth to make shelf/platform",
            default=False
            )
    ShelfX = FloatProperty(
            name="Left",
            description="The x position of Shelf",
            default=-5.00,
            min=-100, max=100.0
            )
    ShelfZ = FloatProperty(
            name="Bottom",
            description="The z position of Shelf",
            default=10.0,
            min=-100, max=100.0
            )
    ShelfH = FloatProperty(
            name="Height",
            description="The Height of Shelf area",
            default=1.0,
            min=0.01, max=100.0
            )
    ShelfW = FloatProperty(
            name="Width",
            description="The Width of shelf area",
            default=5.0,
            min=0.01, max=100.0
            )
    ShelfD = FloatProperty(
            name="Depth",
            description="Depth of each block for shelf (from cursor + 1/2 wall depth)",
            default=2.0,
            min=0.01, max=100.0
            )
    ShelfBack = BoolProperty(
            name="Backside",
            description="Shelf on backside of wall",
            default=False
            )
    # properties for steps (extend blocks in area,  progressive width)
    StepTog = BoolProperty(
            name="Steps",
            description="Add blocks in area by depth with progressive width to make steps",
            default=False
            )
    StepX = FloatProperty(
            name="Left",
            description="The x position of steps",
            default=-9.00,
            min=-100, max=100.0
            )
    StepZ = FloatProperty(
            name="Bottom",
            description="The z position of steps",
            default=0.0,
            min=-100, max=100.0
            )
    StepH = FloatProperty(
            name="Height",
            description="The Height of step area",
            default=10.0,
            min=0.01, max=100.0
            )
    StepW = FloatProperty(
            name="Width",
            description="The Width of step area",
            default=8.0,
            min=0.01, max=100.0
            )
    StepD = FloatProperty(
            name="Depth",
            description="Depth of each block for steps (from cursor + 1/2 wall depth)",
            default=1.0,
            min=0.01, max=100.0
            )
    StepV = FloatProperty(
            name="Riser",
            description="Height of each step",
            default=0.70,
            min=0.01, max=100.0
            )
    StepT = FloatProperty(
            name="Tread",
            description="Width of each step",
            default=1.0,
            min=0.01, max=100.0
            )
    StepLeft = BoolProperty(
            name="Direction",
            description="If checked, flip steps direction towards the -X axis",
            default=False
            )
    StepOnly = BoolProperty(
            name="Steps Only",
            description="Steps only, no supporting blocks",
            default=False
            )
    StepBack = BoolProperty(
            name="Backside",
            description="Steps on backside of wall",
            default=False
            )

    # Display the toolbox options
    def draw(self, context):
        layout = self.layout

        box = layout.box()
        box.prop(self, 'ConstructTog')

        # Wall area (size/position)
        box = layout.box()
        box.label(text="Wall Size (area)")

        col = box.column(align=True)
        col.prop(self, "WallStart")
        col.prop(self, "WallEnd")

        col = box.column(align=True)
        col.prop(self, "WallBottom")
        col.prop(self, "WallTop")
        box.prop(self, "EdgeOffset")

        # Wall block sizing
        box = layout.box()
        box.label(text="Block Sizing")
        box.prop(self, "MergeBlock")

        # add checkbox for "fixed" sizing (ignore variance) a.k.a. bricks
        col = box.column(align=True)
        col.prop(self, "Width")
        col.prop(self, "WidthVariance")
        col.prop(self, "WidthMinimum")

        col = box.column(align=True)
        col.prop(self, "Height")
        col.prop(self, "HeightVariance")
        col.prop(self, "HeightMinimum")

        col = box.column(align=True)
        col.prop(self, "Depth")
        col.prop(self, "DepthVariance")
        col.prop(self, "DepthMinimum")

        # grout settings
        box = layout.box()
        box.label(text="Grout")

        col = box.column(align=True)
        col.prop(self, "Grout")
        col.prop(self, "GroutVariance")

        col = box.column(align=True)
        col.prop(self, "GroutDepth")
        col.prop(self, "GroutDepthVariance")

        # Wall shape modifiers
        box = layout.box()
        box.label(text="Wall Shape")
        row = box.row(align=True)
        row.prop(self, "RadialTog", toggle=True)
        row.prop(self, "SlopeTog", toggle=True)

        # Openings (doors, windows; arched)
        box = layout.box()
        box.prop(self, 'Opening1Tog')
        if self.Opening1Tog:
            col = box.column(align=True)
            col.prop(self, "Opening1Width")
            col.prop(self, "Opening1Height")
            col.prop(self, "Opening1X")
            col.prop(self, "Opening1Z")
            col.prop(self, "Opening1Bevel")

            box.prop(self, "Opening1Repeat", toggle=True)

            sub_box = box.box()
            sub_box.prop(self, "Opening1TopArchTog")
            if self.Opening1TopArchTog:
                col = sub_box.column(align=True)
                col.prop(self, "Opening1TopArch")
                col.prop(self, "Opening1TopArchThickness")

            sub_box = box.box()
            sub_box.prop(self, "Opening1BtmArchTog")
            if self.Opening1BtmArchTog:
                col = sub_box.column(align=True)
                col.prop(self, "Opening1BtmArch")
                col.prop(self, "Opening1BtmArchThickness")

        # Slots (narrow openings)
        box = layout.box()
        box.prop(self, "SlotTog")
        if self.SlotTog:
            col = box.column(align=True)
            col.prop(self, "SlotX")
            col.prop(self, "SlotGap")

            box.prop(self, "SlotRpt", toggle=True)

            sub_box = box.box()
            sub_box.prop(self, "SlotV")
            if self.SlotV:
                col = sub_box.column(align=True)
                col.prop(self, "SlotVH")
                col.prop(self, "SlotVBtm")

            sub_box = box.box()
            sub_box.prop(self, "SlotH")
            if self.SlotH:
                col = sub_box.column(align=True)
                col.prop(self, "SlotHW")
                col.prop(self, "SlotHBtm")

        # Crenels, gaps in top of wall
        box = layout.box()
        box.prop(self, "CrenelTog")
        if self.CrenelTog:
            col = box.column(align=True)
            col.prop(self, "CrenelXP")
            col.prop(self, "CrenelZP")

        # Shelfing (protrusions)
        box = layout.box()
        box.prop(self, 'ShelfTog')
        if self.ShelfTog:
            col = box.column(align=True)
            col.prop(self, "ShelfX")
            col.prop(self, "ShelfZ")

            col = box.column(align=True)
            col.prop(self, "ShelfW")
            col.prop(self, "ShelfH")
            col.prop(self, "ShelfD")

            box.prop(self, "ShelfBack")

        # Steps
        box = layout.box()
        box.prop(self, 'StepTog')
        if self.StepTog:
            col = box.column(align=True)
            col.prop(self, "StepX")
            col.prop(self, "StepZ")

            col = box.column(align=True)
            col.prop(self, "StepH")
            col.prop(self, "StepW")
            col.prop(self, "StepD")

            col = box.column(align=True)
            col.prop(self, "StepV")
            col.prop(self, "StepT")

            col = box.column(align=True)
            row = col.row(align=True)
            row.prop(self, "StepLeft", toggle=True)
            row.prop(self, "StepOnly", toggle=True)
            col.prop(self, "StepBack", toggle=True)

    # Respond to UI - get the properties set by user.
    # Check and process UI settings to generate masonry

    def execute(self, context):
        global radialized
        global slope
        global openingSpecs
        global bigBlock
        global shelfExt
        global stepMod
        global stepLeft
        global shelfBack
        global stepOnly
        global stepBack

        # Create the wall when enabled (skip regen iterations when off)
        if not self.ConstructTog:
            return {'FINISHED'}

        # enter the settings for the wall dimensions (area)
        # start can't be zero - min/max don't matter [if max less than end] but zero don't workie.
        # start can't exceed end.
        if not self.WallStart or self.WallStart >= self.WallEnd:
            self.WallStart = NOTZERO  # Reset UI if input out of bounds...

        dims['s'] = self.WallStart
        dims['e'] = self.WallEnd
        dims['b'] = self.WallBottom
        dims['t'] = self.WallTop

        settings['eoff'] = self.EdgeOffset

        # retrieve the settings for the wall block properties
        settings['w'] = self.Width
        settings['wv'] = self.WidthVariance
        settings['wm'] = self.WidthMinimum

        if not radialized:
            settings['sdv'] = settings['w']
        else:
            settings['sdv'] = 0.12

        settings['h'] = self.Height
        settings['hv'] = self.HeightVariance
        settings['hm'] = self.HeightMinimum

        settings['d'] = self.Depth
        settings['dv'] = self.DepthVariance
        settings['dm'] = self.DepthMinimum

        if self.MergeBlock:
            bigBlock = 1
        else:
            bigBlock = 0

        settings['g'] = self.Grout
        settings['gv'] = self.GroutVariance
        settings['gd'] = self.GroutDepth
        settings['gdv'] = self.GroutDepthVariance

        if self.GroutEdge:
            settings['ge'] = 1
        else:
            settings['ge'] = 0

        # set wall shape modifiers
        if self.RadialTog:
            radialized = 1
            # eliminate to allow user control for start/completion?
            dims['s'] = 0.0       # complete radial
            if dims['e'] > PI * 2:
                dims['e'] = PI * 2  # max end for circle
            if dims['b'] < settings['g']:
                dims['b'] = settings['g']  # min bottom for grout extension
        else:
            radialized = 0

        if self.SlopeTog:
            slope = 1
        else:
            slope = 0

        shelfExt = 0
        shelfBack = 0

        # Add shelf if enabled
        if self.ShelfTog:
            shelfExt = 1
            shelfSpecs['h'] = self.ShelfH
            shelfSpecs['w'] = self.ShelfW
            shelfSpecs['d'] = self.ShelfD
            shelfSpecs['x'] = self.ShelfX
            shelfSpecs['z'] = self.ShelfZ

            if self.ShelfBack:
                shelfBack = 1
        stepMod = 0
        stepLeft = 0
        stepOnly = 0
        stepBack = 0

        # Make steps if enabled
        if self.StepTog:
            stepMod = 1
            stepSpecs['x'] = self.StepX
            stepSpecs['z'] = self.StepZ
            stepSpecs['h'] = self.StepH
            stepSpecs['w'] = self.StepW
            stepSpecs['d'] = self.StepD
            stepSpecs['v'] = self.StepV
            stepSpecs['t'] = self.StepT

            if self.StepLeft:
                stepLeft = 1

            if self.StepOnly:
                stepOnly = 1

            if self.StepBack:
                stepBack = 1

        # enter the settings for the openings
        # when openings overlap they create inverse stonework - interesting but not the desired effect :)
        # if opening width == indent * 2 the edge blocks fail (row of blocks cross opening) - bug.
        openingSpecs = []
        openingIdx = 0  # track opening array references for multiple uses

        # general openings with arch options - can be windows or doors.
        if self.Opening1Tog:
            # set defaults...
            openingSpecs += [{'w': 0.5, 'h': 0.5, 'x': 0.8, 'z': 2.7, 'rp': 1,
                              'b': 0.0, 'v': 0, 'vl': 0, 't': 0, 'tl': 0}]

            openingSpecs[openingIdx]['w'] = self.Opening1Width
            openingSpecs[openingIdx]['h'] = self.Opening1Height
            openingSpecs[openingIdx]['x'] = self.Opening1X
            openingSpecs[openingIdx]['z'] = self.Opening1Z
            openingSpecs[openingIdx]['rp'] = self.Opening1Repeat

            if self.Opening1TopArchTog:
                openingSpecs[openingIdx]['v'] = self.Opening1TopArch
                openingSpecs[openingIdx]['t'] = self.Opening1TopArchThickness

            if self.Opening1BtmArchTog:
                openingSpecs[openingIdx]['vl'] = self.Opening1BtmArch
                openingSpecs[openingIdx]['tl'] = self.Opening1BtmArchThickness

            openingSpecs[openingIdx]['b'] = self.Opening1Bevel

            openingIdx += 1  # count window/door/arch openings

        # Slots (narrow openings)
        if self.SlotTog:

            if self.SlotV:  # vertical slots
                # set defaults...
                openingSpecs += [{'w': 0.5, 'h': 0.5, 'x': 0.0, 'z': 2.7, 'rp': 0,
                                 'b': 0.0, 'v': 0, 'vl': 0, 't': 0, 'tl': 0}]

                openingSpecs[openingIdx]['w'] = self.SlotGap
                openingSpecs[openingIdx]['h'] = self.SlotVH
                openingSpecs[openingIdx]['x'] = self.SlotX
                openingSpecs[openingIdx]['z'] = self.SlotVBtm
                openingSpecs[openingIdx]['rp'] = self.SlotRpt

                # make them pointy...
                openingSpecs[openingIdx]['v'] = self.SlotGap
                openingSpecs[openingIdx]['t'] = self.SlotGap / 2
                openingSpecs[openingIdx]['vl'] = self.SlotGap
                openingSpecs[openingIdx]['tl'] = self.SlotGap / 2

                openingIdx += 1        # count vertical slot openings

            # need to handle overlap of H and V slots...

            if self.SlotH:  # Horizontal slots
                # set defaults...
                openingSpecs += [{'w': 0.5, 'h': 0.5, 'x': 0.0, 'z': 2.7, 'rp': 0,
                                 'b': 0.0, 'v': 0, 'vl': 0, 't': 0, 'tl': 0}]

                openingSpecs[openingIdx]['w'] = self.SlotHW
                openingSpecs[openingIdx]['h'] = self.SlotGap
                openingSpecs[openingIdx]['x'] = self.SlotX
                openingSpecs[openingIdx]['z'] = self.SlotHBtm
                # horizontal repeat isn't same spacing as vertical...
                openingSpecs[openingIdx]['rp'] = self.SlotRpt

                # make them pointy...
                openingIdx += 1  # count horizontal slot openings

        # Crenellations (top row openings)
        if self.CrenelTog:

            # add bottom arch option?
            # perhaps a repeat toggle...
            # if crenel opening overlaps with arch opening it fills with blocks...

            # set defaults...
            openingSpecs += [{'w': 0.5, 'h': 0.5, 'x': 0.0, 'z': 2.7, 'rp': 1,
                              'b': 0.0, 'v': 0, 'vl': 0, 't': 0, 'tl': 0}]

            wallW = self.WallEnd - self.WallStart
            crenelW = wallW * self.CrenelXP  # Width % opening.

            wallH = self.WallTop - self.WallBottom
            crenelH = wallH * self.CrenelZP  # % proportional height.

            openingSpecs[openingIdx]['w'] = crenelW
            openingSpecs[openingIdx]['h'] = crenelH

            # calculate the spacing between openings.
            # this isn't the absolute start (left),
            # it's opening center offset relative to cursor (space between openings)...
            openingSpecs[openingIdx]['x'] = crenelW * 2 - 1  # assume standard spacing

            if not radialized:  # normal wall?
                # set indent 0 (center) if opening is 50% or more of wall width, no repeat.
                if crenelW * 2 >= wallW:
                    openingSpecs[openingIdx]['x'] = 0
                    openingSpecs[openingIdx]['rp'] = 0
            # set bottom of opening (center of hole)
            openingSpecs[openingIdx]['z'] = self.WallTop - (crenelH / 2)

            openingIdx += 1  # count crenel openings

        # Process the user settings to generate a wall
        # generate the list of vertices for the wall...
        verts_array, faces_array = createWall(
                                        radialized, slope, openingSpecs, bigBlock,
                                        shelfExt, shelfBack, stepMod, stepLeft, stepOnly,
                                        stepBack
                                        )

        # Create new mesh
        mesh = bpy.data.meshes.new("Wall")

        # Make a mesh from a list of verts/edges/faces.
        mesh.from_pydata(verts_array, [], faces_array)

        scene = context.scene

        # Deselect all objects.
        bpy.ops.object.select_all(action='DESELECT')

        mesh.update()

        ob_new = bpy.data.objects.new("Wall", mesh)
        scene.objects.link(ob_new)
        # leave this out to prevent 'Tab key" going into edit mode :)
        # Use rmb click to select and still modify.
        scene.objects.active = ob_new
        ob_new.select = True

        ob_new.location = tuple(context.scene.cursor_location)
        ob_new.rotation_quaternion = [1.0, 0.0, 0.0, 0.0]

        return {'FINISHED'}



# gpl author: Ryan Inch (Imaginer)

from bpy.types import Menu
from . import utils_core


class MasterSymmetryMenu(Menu):
    bl_label = "Symmetry Options"
    bl_idname = "VIEW3D_MT_sv3_master_symmetry_menu"

    @classmethod
    def poll(self, context):
        return utils_core.get_mode() in (
                        'SCULPT',
                        'TEXTURE_PAINT'
                        )

    def draw(self, context):
        layout = self.layout

        if utils_core.get_mode() == 'TEXTURE_PAINT':
            layout.row().prop(context.tool_settings.image_paint,
                              "use_symmetry_x", toggle=True)
            layout.row().prop(context.tool_settings.image_paint,
                              "use_symmetry_y", toggle=True)
            layout.row().prop(context.tool_settings.image_paint,
                              "use_symmetry_z", toggle=True)
        else:
            layout.row().menu(SymmetryMenu.bl_idname)
            layout.row().menu(SymmetryRadialMenu.bl_idname)
            layout.row().prop(context.tool_settings.sculpt,
                              "use_symmetry_feather", toggle=True)


class SymmetryMenu(Menu):
    bl_label = "Symmetry"
    bl_idname = "VIEW3D_MT_sv3_symmetry_menu"

    def draw(self, context):
        layout = self.layout

        layout.row().label(text="Symmetry")
        layout.row().separator()

        layout.row().prop(context.tool_settings.sculpt,
                          "use_symmetry_x", toggle=True)
        layout.row().prop(context.tool_settings.sculpt,
                          "use_symmetry_y", toggle=True)
        layout.row().prop(context.tool_settings.sculpt,
                          "use_symmetry_z", toggle=True)


class SymmetryRadialMenu(Menu):
    bl_label = "Radial"
    bl_idname = "VIEW3D_MT_sv3_symmetry_radial_menu"

    def draw(self, context):
        layout = self.layout

        layout.row().label(text="Radial")
        layout.row().separator()

        layout.column().prop(context.tool_settings.sculpt,
                             "radial_symmetry", text="", slider=True)



"""
Preset Menus
++++++++++++

Preset menus are simply a convention that uses a menu sub-class
to perform the common task of managing presets.

This example shows how you can add a preset menu.

This example uses the object draw options,
however you can use properties defined by your own scripts too.
"""

import bpy
from bpy.types import Operator, Menu
from bl_operators.presets import AddPresetBase


class OBJECT_MT_draw_presets(Menu):
    bl_label = "Object Draw Presets"
    preset_subdir = "object/draw"
    preset_operator = "script.execute_preset"
    draw = Menu.draw_preset


class AddPresetObjectDraw(AddPresetBase, Operator):
    '''Add a Object Draw Preset'''
    bl_idname = "camera.object_draw_preset_add"
    bl_label = "Add Object Draw Preset"
    preset_menu = "OBJECT_MT_draw_presets"

    # variable used for all preset values
    preset_defines = [
        "obj = bpy.context.object"
        ]

    # properties to store in the preset
    preset_values = [
        "obj.draw_type",
        "obj.show_bounds",
        "obj.draw_bounds_type",
        "obj.show_name",
        "obj.show_axis",
        "obj.show_wire",
        ]

    # where to store the preset
    preset_subdir = "object/draw"


# Draw into an existing panel
def panel_func(self, context):
    layout = self.layout

    row = layout.row(align=True)
    row.menu(OBJECT_MT_draw_presets.__name__, text=OBJECT_MT_draw_presets.bl_label)
    row.operator(AddPresetObjectDraw.bl_idname, text="", icon='ZOOMIN')
    row.operator(AddPresetObjectDraw.bl_idname, text="", icon='ZOOMOUT').remove_active = True


classes = (
    OBJECT_MT_draw_presets,
    AddPresetObjectDraw,
    )


def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    bpy.types.OBJECT_PT_display.prepend(panel_func)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
    bpy.types.OBJECT_PT_display.remove(panel_func)


if __name__ == "__main__":
    register()
